package com.anabas.ibus;

import com.anabas.concepts.*;
import com.anabas.gxo.*;
import com.anabas.util.misc.*;

import ch.softwired.jms.IBusTopicRequestor;
import ch.softwired.jms.IBusJMSContext;
import javax.jms.*;
/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author
 * @version 1.0
 */

public class GMS_PublisherImpl implements GMS_StreamPublisher {

  private TopicSession   m_session;
  private TopicPublisher m_publisher;
  private GMS_StreamImpl m_parentStream;
  private IBusTopicRequestor m_requestor;

  /**
   * Used by request/response.   request() creates a temporary subscriber which
   * will take on this basic selector attributes....currently basic selector
   * only weed out messages that itself sent since noLocal is not working.
   */
  private GMS_MessageSelector m_baseSelector = new GMS_BaseMessageSelector();

  public GMS_PublisherImpl(TopicPublisher publisher,TopicSession s,GMS_StreamImpl parent) {
    m_publisher = publisher;
    m_parentStream = parent;
    try {
      m_publisher.setDeliveryMode(javax.jms.DeliveryMode.NON_PERSISTENT);
    } catch (JMSException e) {
      LogManager.err("Publisher","Unable to set non persistent mode",e);
    }
    m_session   = s;
  }

  public GMS_Message createMessage(short msgType, short bodyType) throws GXO_Exception {
    GMS_Message msg = null;

    try {
      switch (bodyType) {
        case GMS_Message.BODYTYPE_BYTES:
          msg = new GMS_BytesMessageImpl(m_session.createBytesMessage());
          break;
        case GMS_Message.BODYTYPE_MAP:
          throw new GXO_MsgTypeUnsupportedException(msgType);
        case GMS_Message.BODYTYPE_OBJECT:
          msg = new GMS_ObjectMessageImpl(m_session.createObjectMessage());
          break;
        case GMS_Message.BODYTYPE_TEXT:
          msg = new GMS_TextMessageImpl(m_session.createTextMessage());
          break;
        case GMS_Message.BODYTYPE_XML:
          throw new GXO_MsgTypeUnsupportedException(msgType);
      }

      msg.setProperty("gxo_message_type", String.valueOf(msgType));

      // Set this property so clients can select on this and choose to ignore
      // its own messages.  This is really a work around to IBus's noLocal not
      // working.....I think.
      msg.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,
                      IBusJMSContext.getTopicConnection().getClientID());
    } catch (JMSException e) {
      LogManager.err("GMS_Publisher","Unable to create message",e);
    }

    return msg;
  }

  // Convert a GMS_Message to a JMS Message
  public static Message convertMessage(GMS_Message m) throws GXO_Exception {
    Message msg;

    switch((int)m.getBodyType()) {
      case GMS_Message.BODYTYPE_TEXT:
        msg = ((GMS_TextMessageImpl)m).getJMSMessage();
        break;
      case GMS_Message.BODYTYPE_OBJECT:
        msg = ((GMS_ObjectMessageImpl)m).getJMSMessage();
        break;
      case GMS_Message.BODYTYPE_BYTES:
        msg = ((GMS_BytesMessageImpl)m).getJMSMessage();
        break;
      default:
        throw new GXO_MsgBodyUnsupportedException(m);
    }
    return msg;
  }

  /**
   * Requests a message and block on the first reply.  Requests currently has
   * noLocal set to true so that local client can not reply to its own request.
   *
   * @return null if timeout occurs.
   */
  public GMS_Message request(GMS_Message m, GMS_Destination d,long timeout) throws GXO_Exception {
    try {
      Topic t = m_session.createTemporaryTopic();
      TopicSubscriber s = m_session.createSubscriber(t,m_baseSelector.toString(),true);
      Message msg = convertMessage(m);

      // Tell the listener the reply to temporary topic.
      msg.setJMSReplyTo(t);
      sendMessage(m,d,true);

      //String  dstStr = d.toString();
      // Set the destination list so that only the appropriate subscribers
      // will receive them.
      //m.setProperty(GXO_JMSConstants.GXO_DESTINATION_PROPERTY,dstStr);
      //m_publisher.publish(msg);
      //LogManager.log(LogManager.ALL,"Publisher","Sent request [dst="+dstStr+"] message: " + msg);

      if (timeout == 0) {
        msg = s.receive();
      } else {
        msg = s.receive(timeout);
      }

      if (msg != null) {
        LogManager.log(LogManager.ALL,"Publisher","Got response [correl="+
                      msg.getJMSCorrelationID()+"] message: " + msg);
      } else {
        LogManager.log(LogManager.ALL,"Publisher","Response timed out on " + msg);
        return null;
      }
      return GMS_SubscriberImpl.convertMessage(msg);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  /**
   * Sends a message to a destination.
   */
  public void sendMessage(GMS_Message m, GMS_Destination d,boolean noLocal) throws GXO_Exception {
    Message msg = convertMessage(m);
    String dstString = ((GMS_DestinationImpl)d).getDestinationString();

    m.setProperty(GXO_JMSConstants.GXO_DESTINATION_PROPERTY,dstString);

    try {
      // Set this property so clients can select on this and choose to ignore
      // its own messages.  This is really a work around to IBus's noLocal not
      // working.....I think.
      if (noLocal) {
        m.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,IBusJMSContext.getTopicConnection().getClientID());
      } else {
        // Allow subscribers to receive this message.
        m.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,"");
      }

      m_publisher.publish(msg);
      // if it is a text message, log the message ...
      if(m.getBodyType() == GMS_Message.BODYTYPE_TEXT) {
        LogManager.log(LogManager.ALL,"Publisher",
            "Sent Message [dst="+dstString+",noLocal="+noLocal+",replyTo="+
            msg.getJMSReplyTo()+"]: " + m + " to [" + dstString + "]" );
      }
    } catch (JMSException e) {
      throw new GXO_Exception();
    }
  }

  /**
   * Sends a message to a destination.  The local subscribers will also
   * receive the message if the exthe destination includes the local subscriber.
   */
  public void sendMessage(GMS_Message m, GMS_Destination d) throws GXO_Exception {
    Message msg = convertMessage(m);
    String dstString = ((GMS_DestinationImpl)d).getDestinationString();

    m.setProperty(GXO_JMSConstants.GXO_DESTINATION_PROPERTY,dstString);

    try {
      // Set this property so clients can select on this and choose to ignore
      // its own messages.  This is really a work around to IBus's noLocal not
      // working.....I think.
      m.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,"");

      m_publisher.publish(msg);
      // if it is a text message, log the message ...
      if(m.getBodyType() == GMS_Message.BODYTYPE_TEXT) {
        LogManager.log(LogManager.ALL,"Publisher","Sent Message " + m + " to [" + dstString + "]" );
      }
    } catch (JMSException e) {
      throw new GXO_Exception();
    }
  }

  /**
   * Deliver a broadcast.  The local client will also receive the broadcasted message.
   */
  public void broadcast(GMS_Message m) throws GXO_Exception {
    Message msg = convertMessage(m);

    m.setProperty(GXO_JMSConstants.GXO_DESTINATION_PROPERTY,
                  GXO_JMSConstants.GXO_DESTINATION_ALL);

    try {
      // Set this property so clients can select on this and choose to ignore
      // its own messages.  This is really a work around to IBus's noLocal not
      // working.....I think.
      m.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,"");

      m_publisher.publish(msg);
      // if it is a text message, log the message ...
      if(m.getBodyType() == GMS_Message.BODYTYPE_TEXT) {
        LogManager.log(LogManager.ALL,"Publisher","Broadcast message " + m);
      }
    } catch (JMSException e) {
      throw new GXO_Exception();
    }
  }

  /**
   * Broadcasts a message with explicit noLocal.  If noLocal is true then the
   * message is not delivered to the local subscribers.
   */
  public void broadcast(GMS_Message m,boolean noLocal) throws GXO_Exception {
    Message msg = convertMessage(m);

    m.setProperty(GXO_JMSConstants.GXO_DESTINATION_PROPERTY,
                  GXO_JMSConstants.GXO_DESTINATION_ALL);

    try {
      // Set this property so clients can select on this and choose to ignore
      // its own messages.  This is really a work around to IBus's noLocal not
      // working.....I think.
      if (noLocal) {
        m.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,IBusJMSContext.getTopicConnection().getClientID());
      } else {
        // Allow subscribers to receive this message.
        m.setProperty(GXO_JMSConstants.IBUS_CLIENTID_PROPERTY,"");
      }

      m_publisher.publish(msg);
      // if it is a text message, log the message ...
      if(m.getBodyType() == GMS_Message.BODYTYPE_TEXT) {
        LogManager.log(LogManager.ALL,"Publisher","Broadcast message " + m);
      }
    } catch (JMSException e) {
      throw new GXO_Exception();
    }
  }
}
