package com.anabas.ibus;

import javax.jms.*;
import ch.softwired.jms.*;
import java.util.Hashtable;

import com.anabas.util.misc.*;
import com.anabas.gxo.*;
/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author
 * @version 1.0
 */

public class GMS_StreamFactoryImpl implements GMS_StreamFactory {

  protected TopicSession   m_topicSession = null;
  protected GMS_MessageSelector m_defaultSelector;
  private   String              m_clientID;
  private   String              m_host;
  private   String              m_port;
  private   IBusTopicConnection m_connection;

  /**
   * Sets setup properties for the event bus factory.  Such as connection
   * parameters, authentication parameters, ...
   */
  public void setProperty(String str, Object obj) {
    if (str.equals("userid")) {
      m_clientID = (String)obj;
    } else if (str.equals("host")) {
      try {
        m_host = (String)obj;
      } catch (Exception e) {
        LogManager.err("GMS_StreamFactory","Unable to set server host",e);
      }

    } else if (str.equals("port")) {
      try {
        m_port = (String)obj;
      } catch (Exception e) {
        LogManager.err("GMS_StreamFactory","Unable to set server port",e);
      }

    }
  }

  /**
   * Initialize IBus connection.  Invoked internally when the first stream
   * is created.
   */
  private void initIBus() throws JMSException {
    LogManager.log("GMS_StreamFactory","Initializing bus:");
    IBusTopicConnectionFactory factory = (IBusTopicConnectionFactory)IBusJMSContext.getTopicConnectionFactory();
    if (m_host != null) {
      factory.setServerName(m_host);
      LogManager.log("GMS_StreamFactory","  - server: "+m_host);
    }

    if (m_port != null) {
      factory.setServerPort(m_port);
      LogManager.log("GMS_StreamFactory","  - port: "+m_port);
    }


    m_connection = (IBusTopicConnection)factory.createTopicConnection();
    if (m_clientID != null) {
      LogManager.log("GMS_StreamFactory","  - clientID: "+m_clientID);
      m_connection.setClientID(m_clientID);
    }

    IBusContext.s_defaultSelector = m_defaultSelector = new GMS_UserSelector(m_clientID);
  }

  /**
   * Creates a new stream within the stream context.  A stream is the leaf node
   * of a stream tree.  Basically a DirContext.bind(name,GMS_Collection) operation.
   *
   * @param params A list of parameters for the stream.  For example:
   *        "min_latency" - "50ms"
   *        "encrypted"   - "true"
   *        "persistent"  - "none"
   */

  /**
   * Creates a stream with a default selector set in the context factory.
   */
  public GMS_Stream createStream(String name, Hashtable params) throws GXO_AccessDeniedException {
    if (m_topicSession == null) {
      try {
        if (m_connection == null) initIBus();
        m_topicSession = m_connection.createTopicSession(false,TopicSession.AUTO_ACKNOWLEDGE);
      } catch (JMSException e) {
        LogManager.err("GMS_StreamFactory","Unable to initialize IBus connection",e);
      }
    }
    return new GMS_StreamImpl("",name,m_topicSession);
  }
}