package com.anabas.sdsharedlet;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;

import com.anabas.gxo.*;
import com.anabas.concepts.*;
import com.anabas.sharedlet.*;
import com.anabas.naming.*;
import com.anabas.util.misc.LogManager;
import com.anabas.sharedlet.SessionManager;

/**
 * Title:        Shared Display Sharedlet
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Anabas Inc.
 * @author Milind
 * @version 1.0
 */

class SDEventListenerThread implements Runnable {
  /* Native Event Definitions */
  private static final int WINDOW_EVENTS_BASE = 0;

  private static final int WE_ACTIVATED   = WINDOW_EVENTS_BASE + 0;
  private static final int WE_CLOSED      = WINDOW_EVENTS_BASE + 1;
  private static final int WE_CLOSING     = WINDOW_EVENTS_BASE + 2;
  private static final int WE_DEACTIVATED = WINDOW_EVENTS_BASE + 3;
  private static final int WE_DEICONIFIED = WINDOW_EVENTS_BASE + 4;
  private static final int WE_ICONIFIED   = WINDOW_EVENTS_BASE + 5;
  private static final int WE_OPENED      = WINDOW_EVENTS_BASE + 6;
  private static final int WE_HIDDEN      = WINDOW_EVENTS_BASE + 7;
  private static final int WE_MOVED       = WINDOW_EVENTS_BASE + 8;
  private static final int WE_RESIZED     = WINDOW_EVENTS_BASE + 9;
  private static final int WE_SHOWN       = WINDOW_EVENTS_BASE + 10;
  private static final int WE_ENABLE_ANNOTATION = WINDOW_EVENTS_BASE + 11;

  private static final int DISPLAYUPDATE_EVENT  = 1;
  private static final int FULL_UPDATE_EVENT    = 2;
  private static final int MOUSEUPDATE_EVENT    = 3;
  private static final int ANNOTATION_EVENT     = 6;
  private static final int END_ANNOTATION_EVENT = 7;
  private static final int WINDOW_EVENT         = 9;

  static final int RFU_USER_REQUEST = 102;
  private static final int RFU_THREAD_REQUEST = 101;
  private static final int RFU_BROADCAST = 0;

  /* Thread State Constants */
  private static final int THREAD_STATE_NOT_STARTED   = 0;
  private static final int THREAD_STATE_RUNNING       = 1;
  private static final int THREAD_STATE_SUSPENDED     = 2;
  private static final int THREAD_STATE_STOPPED       = 3;

  /* Miscellaneous Constants */
  private static final int RFU_FREQ = 3 * 60 * 1000;  // 3 min.
  private static final int MAX_WAIT_TIME = 2000; // in milliseconds
  private static final String RFU_MAJOR_EVENT_PROPERTY = "diff";
  private static final String RFU_MAJOR_EVENT_VAL = "diff_val";

  /* Instance variables */
  private GMS_Stream              m_stream;
  private GMS_StreamPublisher     m_publisher;
  private Thread thread;
  private int m_threadState;
  private SDWrapper m_sdWrapper;
  private SDPresentationView m_presentationView;
  private SDControlView m_controlView;
  private RFUThread m_RFUThread;
  private SDNativeRenderer m_nativeRenderer;
  static private int m_count;
  private EventLog m_eventLog;
  private boolean m_bRFURequested = false;
  private SharedletInfo m_sharedletInfo;
  private SharedletViewManager m_viewManager;

  /* Inner class */
  private class RFUThread implements Runnable {
    private Thread m_thread2;
    private boolean bStopThread;

    RFUThread() {
      bStopThread = true;
    }

    public void run() {
      log("RFU thread started ...");
      int TimeSinceRFU = 0;
      while(true) {
        if(bStopThread)
	{
		break;
	}
        try {
	    Thread.sleep(100);
	    if( TimeSinceRFU < RFU_FREQ )
	    {
		TimeSinceRFU += 100;
	    }
	    else
	    {
		requestFullUpdate(RFU_THREAD_REQUEST);
		TimeSinceRFU = 0;
	    }

        }
        catch (Exception ex) {
        }
      }
    }

    public void beginThread() {
      m_thread2 = new Thread(this);
      bStopThread = false;
      m_thread2.start();
    }

    public void stopThread() {
      log("Stopping RFU thread ...");
      bStopThread = true;
    }
  };

  /* Constructor */
  public SDEventListenerThread(GMS_StreamPublisher publisher, GMS_Stream stream,
                  SDWrapper sdWrapper, SDPresentationView presentationView, SDControlView controlView) {
    // Initialize the instance variables
    m_stream = stream;
    m_publisher = publisher;
    m_threadState = THREAD_STATE_NOT_STARTED;
    //log("Creating SDWrapper from thread ...");
    m_sdWrapper = sdWrapper;
    m_presentationView = presentationView;
    m_controlView = controlView;

    m_nativeRenderer = (SDNativeRenderer) presentationView.getRenderer(
                                  SDPresentationView.JAVA_RENDERER_MIME);

    m_RFUThread = new RFUThread();
    //m_eventLog = new EventLog();
  }

  public void run() {
    log("thread started ...");
    while(true) {
      switch (m_threadState) {
        case THREAD_STATE_RUNNING:
          break;
        case THREAD_STATE_SUSPENDED:
          synchronized (this) {
            try {
              wait();
            }
            catch (InterruptedException ex) {
              log("interrupted while waiting ...");
            }
          }
          // continue to check whether it is stoped or resumed ...
          continue;
        case THREAD_STATE_STOPPED:
          cleanUpThread();
          return;
      }

      //log(">>>>>>>>>>>>>> waiting for queued events ...");
      // if one or more events are there ...
      if(m_sdWrapper.waitForEvents(MAX_WAIT_TIME)) {
       // get all events and broadcast them on the Event Bus ...
        getAllEvents();
      } else {
        //log(">>>>>>>>>>>>> No events till now ....");
      }
    }
  }

  private void getAllEvents() {
    int EventType;
    byte[] buf = null;

    while(true) {
//log("Getting quedud event ...");
      // get next queued event from the native code ...
      buf = m_sdWrapper.getQueuedEvent();

      // check if there are no more events ...
      if(buf[0] == 0)
        break;

      // Extract event type
      // The first four bytes from the second byte is the Event Type ...
      EventType = buf[1];

      /*if(EventType != MOUSEUPDATE_EVENT && EventType != WINDOW_EVENT)
        log("Event from Native code --> [" + EventType + "]");*/

      switch (EventType) {
        case WINDOW_EVENT:
          processWindowEvents(buf);
          break;

        case END_ANNOTATION_EVENT:
          m_controlView.endAnnotation();
          break;

        case FULL_UPDATE_EVENT:
          processFullUpdateEvent(buf);
          break;

        default:
          // create & broadcast message ...
          broadcastBytesMessage(buf,1,buf.length-1);
          break;
      }
    }
  }

  private void processWindowEvents(byte[] buf) {
    //log("processWindowEvents: called ...");
    Component comp = new Button();
    Window wnd = new Frame();
    int windowEvent = SDNativeRenderer.byteArrToInt(buf,2);

    /*
    StringBuffer sb = new StringBuffer(120);
    sb.append("Window Event: ");
    for (int i = 0; i < buf.length; i++) {
      sb.append("[" + buf[i] + "] ");
    }
    elog(sb.toString());
    */

    switch (windowEvent) {
      /*case WE_ENABLE_ANNOTATION:
        int iEnable = SDNativeRenderer.byteArrToInt(buf,6);
        m_controlView.enableAnnotation(iEnable != 0);
        break;*/

      case WE_ACTIVATED:
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_ACTIVATED),buf);
        break;

      case WE_CLOSED:
        deactivateControlView();
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_CLOSED),buf);
        break;

      case WE_CLOSING:
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_CLOSING),buf);
        break;

      case WE_DEACTIVATED:
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_DEACTIVATED),buf);
        break;

      case WE_DEICONIFIED:
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_DEICONIFIED),buf);
        break;

      case WE_ICONIFIED:
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_ICONIFIED),buf);
        break;

      case WE_OPENED:
        m_nativeRenderer.setWindowEvent(new WindowEvent(wnd,WindowEvent.WINDOW_OPENED),buf);
        break;

      case WE_HIDDEN:
        m_nativeRenderer.setComponentEvent(new ComponentEvent(comp,ComponentEvent.COMPONENT_HIDDEN),buf);
        break;

      case WE_MOVED:
        m_nativeRenderer.setComponentEvent(new ComponentEvent(comp,ComponentEvent.COMPONENT_MOVED),buf);
        break;

      case WE_RESIZED:
        m_nativeRenderer.setComponentEvent(new ComponentEvent(comp,ComponentEvent.COMPONENT_RESIZED),buf);
        break;

      case WE_SHOWN:
        m_nativeRenderer.setComponentEvent(new ComponentEvent(comp,ComponentEvent.COMPONENT_SHOWN),buf);
        break;

      default:
        log("processWindowEvents: got unknown Window Event from native DLL [" + windowEvent + "]");
        break;
    }
  }

  synchronized void requestFullUpdate(int requestId) {
    if(m_bRFURequested)
      return;
    m_bRFURequested = true;
    m_sdWrapper.RequestFullUpdate(requestId);
  }

  synchronized private void processFullUpdateEvent(byte[] buf) {
    log("<<<<<<<<< processFullUpdateEvent: called >>>>>>>>>>>");
    try {
      // make this as major event message
      GMS_BytesMessage msg = (GMS_BytesMessage) m_publisher.createMessage(GMS_Message.MSGTYPE_MAJOR,
                                                    GMS_Message.BODYTYPE_BYTES);
      if(msg == null) {
        log("processFullUpdateEvent: null message ...");
        SDSharedlet.handleDisconnected();
        return;
      }

      msg.writeBytes(buf,1,buf.length-1);

      int reqId = SDNativeRenderer.byteArrToInt(buf,2);

      /*StringBuffer tmpStr = new StringBuffer("Request ID:");
      for(int i=0;i < 14;i++) {
        tmpStr.append("[" + buf[i] + "] ");
      }
      elog(tmpStr.toString());
      elog("==> [" + reqId + "]");*/

      // If initial update, broadcast it to all participants

      if(reqId == SDSharedlet.PDA_ADAPTOR_FU)
      {
	return;
      }
      else if(reqId == 0)
      {
	m_publisher.broadcast(msg);
      }
      else
      {
        UserID userId = null;
        // send this msg to all users in RFU queue
        while((userId = m_presentationView.getNextInRFUQue()) != null) {
          log("sending Full Update to [" + userId.getID() + "]");

          GMS_UserDestinationSelector userSelector = (GMS_UserDestinationSelector)
                m_publisher.createDestinationSelector("GMS_UserDestinationSelector");
          if(userSelector == null) {
            log("cannot get the selector ...");
            break;
          }
          userSelector.setUserID(userId);
          GMS_Destination dest = m_stream.findDestination(userSelector);
          if(dest != null)
            m_publisher.sendMessage(msg, dest);
          else log("processFullUpdateEvent: destination null ...");
        }
      }
      msg.setProperty(RFU_MAJOR_EVENT_PROPERTY,RFU_MAJOR_EVENT_VAL);

      log("sending it to recorder ...");

      // send this msg to the Recorder
//      m_publisher.broadcast(msg);

      m_bRFURequested = false;
    }
    catch (Exception ex)
    {
      SDSharedlet.handleDisconnected();
      err("processFullUpdateEvent: Error",ex);
    }
    //log("<<<<<<<<< processFullUpdateEvent: returning >>>>>>>>>>>");
  }

  public void beginThread(boolean bStartRFUThread) {
    // Create the thread object ...
    thread = new Thread(this);
    // set the thread stare to THREAD_STATE_RUNNING
    m_threadState = THREAD_STATE_RUNNING;
    // start the thread
    thread.start();
    //log("thread started ...");
    if(bStartRFUThread)
      m_RFUThread.beginThread();
  }

  public void suspendThread() {
    // set the thread stare to THREAD_STATE_RUNNING
    m_threadState = THREAD_STATE_SUSPENDED;
  }

  public void resumeThread() {
    // set the thread stare to THREAD_STATE_RUNNING
    m_threadState = THREAD_STATE_RUNNING;
    // notify this thread to resume ...
    synchronized (this) {
      notify();
    }
  }

  public void stopThread() {
    log("stopping the thread ...");
    // if the thread is suspended first resume it ...
    if(m_threadState == THREAD_STATE_SUSPENDED) {
      // set the thread stare to THREAD_STATE_RUNNING
      m_threadState = THREAD_STATE_STOPPED;
      synchronized (this) {
        notify();
      }
    }
    // set the thread stare to THREAD_STATE_RUNNING
    m_threadState = THREAD_STATE_STOPPED;
    //m_eventLog.closeLog();
    m_RFUThread.stopThread();
  }

  public int getState() {
    return m_threadState;
  }

  void startRFUThread(boolean start) {
    if(start)
      m_RFUThread.beginThread();
    else
      m_RFUThread.stopThread();
  }

  private void cleanUpThread() {
  }

  private void deactivateControlView() {
    log("deactivateing the Control View");
    LayoutService layoutService = null;
    try {
      Context ctx = ContextManager.getInitialContext();
      // get the Layout service handle
      layoutService = (LayoutService) ctx.lookup("services/LayoutService");
    }
    catch (Exception ex) {
      err("deactivateControlView: Error ",ex);
    }
    //layoutService.deactivateView(m_controlView);
    layoutService.deactivateViews(SDSharedletInfo.g_sharedletMIME);
  }

  private void broadcastBytesMessage(byte[] baData,int StartIndex,int bytes)
  {
      try {
        GMS_BytesMessage msg = (GMS_BytesMessage) m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,
                                                      GMS_Message.BODYTYPE_BYTES);

        if(msg == null) {
          log("broadcastBytesMessage: null message ...");
          SDSharedlet.handleDisconnected();
          return;
        }
        msg.writeBytes(baData,StartIndex,bytes);

	// true for no local copy of the msg
//	log("broadcastBytesMessage: " + bytes + "bytes");
        m_publisher.broadcast(msg);
      } catch (Exception ex) {
        SDSharedlet.handleDisconnected();
//      err("broadcastBytesMessage: Error",ex);
      }
  }

  private boolean getMyViewInfo() throws Exception {
    Context ctx = ContextManager.getInitialContext();
    SharedletManager sharedletManager = (SharedletManager) ctx.lookup("services/SharedletManager");

    m_sharedletInfo = sharedletManager.getInfo(SDSharedletInfo.g_sharedletMIME);

    if(m_sharedletInfo == null) {
      throw new Exception("### getMyViewInfo: cannot get sharedlet info for [" +
                  SDSharedletInfo.g_sharedletMIME + "]");
    }
    m_viewManager = sharedletManager.getViewManager();

    return m_viewManager != null;
  }

  SharedletView getMyView(String viewName) throws Exception {
    if((m_sharedletInfo == null || m_viewManager == null) && !getMyViewInfo())
    {
      log("==========>> 1 : " + m_sharedletInfo + ", " + m_viewManager + "<<======");
      return null;
      }

    Vector views = m_sharedletInfo.getViewInfos();
    if(views == null) {
      throw new Exception("### getMyView: cannot get views for [" + SDSharedletInfo.g_sharedletMIME + "]");
    }

    SharedletViewInfo viewInfo = null;
    Enumeration enum = views.elements();
    while (enum.hasMoreElements()) {
      SharedletViewInfo item = (SharedletViewInfo) enum.nextElement();
      if(item.getID().equals(viewName)) {
        viewInfo = item;
        break;
      }
    }

    if(viewInfo == null) {
      throw new Exception("### getMyView: cannot get view named [" + viewName + "]");
    }
    return m_viewManager.getView(viewInfo);
  }


  private void err(String ErrMsg, Exception ex) {
    LogManager.err("SDSharedlet::SDEventListenerThread",ErrMsg,ex);
  }

  void log(String msg) {
    LogManager.log("SDSharedlet::SDEventListenerThread",msg);
    //m_eventLog.log(msg);
  }

  void elog(String msg) {
    //m_eventLog.log(msg);
  }

  class EventLog {
    private static final String DEFAULT_LOG_FILE = "C:\\Documents and Settings\\milind\\Desktop\\SDEventLog.txt";
    private String m_filename;
    private File m_file;
    private PrintWriter m_pw;

    EventLog() {
      this(DEFAULT_LOG_FILE);
    }

    EventLog(String filename) {
      m_filename = filename;
      m_file = new File(m_filename);
      try {
        m_pw = new PrintWriter(new BufferedWriter(new FileWriter(m_filename,true)));
        m_pw.write("------------------- Log -----------------\r\n");
      }
      catch (Exception ex) {
        err("cannot open file for writting [" + m_file + "]", ex);
      }
    }

    void log(String str) {
      m_pw.write(str+"\r\n");
    }

    void closeLog() {
      try {
       m_pw.close();
      }
      catch (Exception ex) {

      }
    }
  }
}