package com.anabas.sdsharedlet;

import java.io.*;
import java.net.*;
import java.util.*;
//import javax.swing.JOptionPane;
import com.anabas.util.ui.AnabasMessageDialog;
import com.anabas.sharedlet.*;
import com.anabas.naming.*;
import com.anabas.util.nativeutils.NativeUtilsWrapper;
import com.anabas.util.misc.LogManager;

/**
 * Title:        Shared Display Sharedlet
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Anabas Inc.
 * @author
 * @version 1.0
 */

public class SDSharedlet extends DefaultSharedletImpl
{
    static private boolean m_bAlreadyDisplayed;
    private SharedletInfo m_sharedletInfo;
    private SharedletViewManager m_viewManager;
    private LayoutService m_layoutService;
    private Vector m_listeners;

  public final static long PDA_ADAPTOR_FU = 0xFFFF0000;


  /* Instance variables */
  private Hashtable m_ht_curURLs = new Hashtable();
  private SDWrapper m_sdWrapper;
  private SDControlView m_controlView;
  private NativeUtilsWrapper m_nativeUtilsWrapper;

  public SDSharedlet() {
    super(SDSharedletInfo.g_sharedletMIME);
    log("Creating SDSharedlet ...");
    m_nativeUtilsWrapper = new NativeUtilsWrapper();
    m_sdWrapper = new SDWrapper();
    m_bAlreadyDisplayed = false;
  }

  public void loadDocument(URL url) {
    int hwnd = 0;

    log("loadDocument: called with [" + url + "]");

    try {
      // activate the control view of the SD sharedlet
      loadSDSharedletViews();
      // get reference to the control view
      getControlView();
    }
    catch (Exception ex) {
      err("Error while loading SDSharedlet views",ex);
    }

    // get the path of actual file on the local File System
    String strFile = getActualFile(url);

    if(strFile == null)
      return;

    File file = new File(strFile);

    if(!file.exists()) {
      err("loadDocument: file doesn't exists [" + strFile + "]",new Exception());
      errMsg("File doesn't exist ...");
      return;
    }

    boolean bIsURL = false;
    String strUrl = null;
    if(strFile.endsWith(".url")) {
      strUrl = getActualURL(file);
      bIsURL = true;
    } else {
      strUrl = url.toString();
    }

    log("loadDocument: loading [" + strUrl + "]");

    /*if(strUrl.endsWith("/"))
      strUrl = strUrl.substring(0,strUrl.length() - 1);*/

    // check if the document is already loaded ....
    if(!bIsURL) {
      Enumeration enum = m_ht_curURLs.keys();
      while (enum.hasMoreElements()) {
        Integer key = (Integer) enum.nextElement();
        String val = (String) m_ht_curURLs.get(key);

        log("Key [" + key + "] Val [" + val + "]");

        if(val.equals(strUrl)) {
          log("loadDocument: document already loaded : [" + strUrl + "]");
          //m_controlView.disableAnnotation();
          m_sdWrapper.ClickToShare(key.intValue(),SDWrapper.CTS_ACTIVATE);
          return;
        }
      }
    }

    log("loadDocument: calling launch with : " + strFile);
    hwnd = m_nativeUtilsWrapper.launch(strFile);
    if(hwnd == 0) {
      log("##########################################");
      log("loadDocument: Could not get new window handle ...");
      log("##########################################");
    } else if(hwnd < 0) {
      err("loadDocument: cannot launch the file",new Exception());
      String ErrMsgs = "Cannot launch the file ...\nNo supported application for this file found ...";
      errMsg(ErrMsgs);
    } else {
      /*if(bIsURL) {
        log("######### launching a URL ... setting Browser wnd in control view ############");
        m_controlView.setBrowserWnd(strUrl);
      }*/
      log("loadDocument: launch successful : window handle [" + hwnd + "]");
      if(m_sdWrapper.ClickToShare(hwnd,SDWrapper.CTS_SHARE)) {
        //m_controlView.disableAnnotation();
        if(!bIsURL)
          m_ht_curURLs.put(new Integer(hwnd),strUrl);
      } else {
        log("loadDocument: could not share the new doc ...");
      }
    }
  }

  String getActualURL(File file) {
    Properties props = new Properties();
    FileInputStream fis = null;

    try {
      fis = new FileInputStream(file);
      props.load(fis);
      Enumeration enum = props.propertyNames();
      while (enum.hasMoreElements()) {
        String prop = (String) enum.nextElement();
        if(prop.equalsIgnoreCase("URL")) {
          return props.getProperty(prop);
        }
      }
    }
    catch (IOException ex) {
      err("getActualURL: Error",ex);
      return "";
    }
    finally {
      try {
        if(fis != null)
          fis.close();
      }
      catch (Exception ex) {
        err("Cannot close file : " + file,ex);
      }
    }
    return "";
  }

  void removeURL(Integer hwnd) {
    m_ht_curURLs.remove(hwnd);
  }

  private boolean getLayoutService() {
    try {
      Context ctx = ContextManager.getInitialContext();
      // get the Layout service handle
      m_layoutService = (LayoutService) ctx.lookup("services/LayoutService");
    }
    catch (Exception ex) {
      //err("getLayoutService: Error ",ex);
      return false;
    }
    return true;
  }

  void loadSDSharedletViews() throws Exception {
    if(m_layoutService != null || getLayoutService()) {
      log(">>>>>>>>>>>> activating Sharedlet views <<<<<<<<<<<<<");
      m_layoutService.activateViews(SDSharedletInfo.g_sharedletMIME);
    } else {
      log("################################");
      log("loadSDSharedletViews: Could not get the LayoutService ...");
      log("################################");
    }
  }

  private void getControlView() throws Exception {
    m_controlView = (SDControlView) getMyView(SDControlViewInfo.g_SDControlViewName);
  }

  private String getActualFile(URL url) {
    // get the scheme from URL
    String strScheme = url.getProtocol();
    //log("Protocol : " + strScheme);

    // get the file path from this url
    String strFile = url.getFile();
    //log("File : " + strFile);

    // get the file separator of the OS
    String fileSeparator = System.getProperty("file.separator");
    strFile = strFile.replace('/',fileSeparator.charAt(0));

    // if file scheme
    if(strScheme.equalsIgnoreCase("file")) {
      String strDriveName = url.getHost();
      //log("Host : " + strDriveName);
      return strDriveName.equals("") ?  strFile
                                     :  strDriveName.charAt(0) + ":" + strFile;
    } else {
        err("getActualFile: not a file url ...",new Exception());
        String ErrMsgs = "Cannot open the URL :" + url.toString();
        errMsg(ErrMsgs);
        return null;
    }
  }
/*  private void copyURL(URL url,File file) {
    try {
      // open URL connection
      URLConnection con = url.openConnection();
      // open the stream
      InputStream is = url.openStream();
      // open this temp file for writting
      FileOutputStream fos = new FileOutputStream( file);
      int ch, bytes = 0;
      while( (ch = is.read()) != -1) {
        fos.write(ch);
        bytes++;
      }

      is.close();
      fos.close();
    }
    catch (Exception ex) {
      LogManager.err("NativeSharedlet","Error copying the URL ...",ex);
    }
  }*/
/*  public static void loadURL() {
     SDSharedlet ns = new SDSharedlet();
    //String file = "http://anabas-server/test/AppletTest.html";
    String file = "file://C:/milind/tmp/b.url";
    //String file = "file://m:/milind/Win32Notes.txt";

    //String file = "file://C:/milind/Readme.txt";
    try {
      URL url = new URL(file);
      ns.loadDocument(url);
    }
    catch (Exception ex) {
    }
  }*/

  private boolean getMyViewInfo() throws Exception {
    Context ctx = ContextManager.getInitialContext();
    SharedletManager sharedletManager = (SharedletManager) ctx.lookup("services/SharedletManager");

    m_sharedletInfo = sharedletManager.getInfo(SDSharedletInfo.g_sharedletMIME);

    if(m_sharedletInfo == null) {
      throw new Exception("### getMyViewInfo: cannot get sharedlet info for [" +
                  SDSharedletInfo.g_sharedletMIME + "]");
    }
    m_viewManager = sharedletManager.getViewManager();

    return m_viewManager != null;
  }

  SharedletView getMyView(String viewName) throws Exception {
    if((m_sharedletInfo == null || m_viewManager == null) && !getMyViewInfo())
    {
      log("==========>> 1 : " + m_sharedletInfo + ", " + m_viewManager + "<<======");
      return null;
      }

    Vector views = m_sharedletInfo.getViewInfos();
    if(views == null) {
      throw new Exception("### getMyView: cannot get views for [" + SDSharedletInfo.g_sharedletMIME + "]");
    }

    SharedletViewInfo viewInfo = null;
    Enumeration enum = views.elements();
    while (enum.hasMoreElements()) {
      SharedletViewInfo item = (SharedletViewInfo) enum.nextElement();
      if(item.getID().equals(viewName)) {
        viewInfo = item;
        break;
      }
    }

    if(viewInfo == null) {
      throw new Exception("### getMyView: cannot get view named [" + viewName + "]");
    }
    return m_viewManager.getView(viewInfo);
  }

  /**
   *
   */
  static public void main(String[] args) throws Exception{
    /*SDSharedletInfo info = new SDSharedletInfo();

    String[] types = info.getSupportedDocumentTypes();
    for (int i = 0; i < types.length; i++) {
      log(types[i]);
    }*/
     SDSharedlet ns = new SDSharedlet();
    //String file = "http://anabas-server/test/AppletTest.html";
    //String file = "file://C:/milind/tmp/x.xml";
    String file1 = "file://C:/milind/tmp/a.url";
    String file2 = "file://C:/milind/tmp/b.url";
    //String file = "file://m:/milind/Win32Notes.txt";

    //String file = "file://C:/milind/Readme.txt";
    try {
      //URL url = new URL(file);
      ns.loadDocument(new URL(file1));
      Thread.sleep(5000);
      ns.log("------------------ Now loading File 2 ---------------");
      ns.loadDocument(new URL(file2));
    }
    catch (Exception ex) {
    }
  }

  static void handleDisconnected() {
    if(!m_bAlreadyDisplayed)
    {
      m_bAlreadyDisplayed = true;
      String ErrMsg = "The connection with the Server is lost\nPlease restart the Application and Try again ...";
      errMsg(ErrMsg);
    }
  }

  static void errMsg(Object ErrMsg) {
    //JOptionPane.showMessageDialog(null, ErrMsg, "Error", JOptionPane.ERROR_MESSAGE);

    //Modified by Dexin 04/12 to get arround Netscape repaint problem.
    AnabasMessageDialog dialog = new AnabasMessageDialog(null, ErrMsg.toString(), "Error", AnabasMessageDialog.ERROR_MESSAGE);
    dialog.show();
  }

  private void err(String ErrMsg, Exception ex) {
    LogManager.err("SDSharedlet::SDSharedlet",ErrMsg,ex);
  }

  private static void log(String msg) {
    LogManager.log("SDSharedlet::SDSharedlet",msg);
  }

    public SDBitmap getBitmap(int format)
    {
        SDBitmap    bitmap = new SDBitmap();
	byte[]      data = m_sdWrapper.getBitmap(format);
	bitmap.format = (int) data[0];
	bitmap.format &= 0x000000ff;
	bitmap.width = SDNativeRenderer.byteArrToInt(data, 1);
	bitmap.height = SDNativeRenderer.byteArrToInt(data, 5);
	bitmap.data = new byte[data.length - 9];

        System.arraycopy(bitmap.data, 0, data, 9, data.length - 9);
	return(bitmap);
    }

    public void addSDEventListener(SDEventListener listener)
    {
        m_listeners.addElement(listener);
    }

    public void removeSDEventListener(SDEventListener listener)
    {
        m_listeners.removeElement(listener);
    }

    public void fireSDEvent_fullUpdate(byte[] fullUpdateMsg)
    {
	Enumeration msgIter = m_listeners.elements();

	while(msgIter.hasMoreElements())
	{
	    SDEventListener listener = (SDEventListener) msgIter.nextElement();
	    listener.onFullUpdate(fullUpdateMsg);
	}
    }
}