package com.anabas.ibus;

import java.io.Serializable;
import java.util.Enumeration;

import javax.jms.BytesMessage;
import javax.jms.Message;
import javax.jms.JMSException;

import com.anabas.concepts.*;
import com.anabas.gxo.GMS_Message;
import com.anabas.gxo.GMS_MessageID;
import com.anabas.gxo.GMS_BytesMessage;
import com.anabas.gxo.GXO_Exception;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author
 * @version 1.0
 *
 * Wrapper message for JMS's BytesMessage
 */

public class GMS_BytesMessageImpl extends GMS_BytesMessage {

  // Used to count length of JMS Message....unfortunately, can't find out
  // the length from BytesMessage it looks like.
  private static byte[] s_counterBytes = new byte[100];

  private BytesMessage m_message;
  private GMS_MessageIDImpl m_id;

  public GMS_BytesMessageImpl(BytesMessage msg) {
    m_message = msg;
    try {
      m_id = new GMS_MessageIDImpl(msg.getJMSMessageID());
    } catch (JMSException e) {
      m_id = new GMS_MessageIDImpl("error");
    }
  }

  public Message getJMSMessage() { return m_message; }

  public void writeByte(byte parm1) throws GXO_Exception {
    try {
      m_message.writeByte(parm1);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public void writeBytes(byte[] parm1) throws GXO_Exception {
    try {
      m_message.writeBytes(parm1);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public void writeBytes(byte[] parm1, int parm2, int parm3) throws GXO_Exception {
    try {
      m_message.writeBytes(parm1,parm2,parm3);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public byte[] getBytes() throws GXO_Exception {
    int length = 0;
    int numBytes;
    byte[] bytes = null;

    try {
      while ((numBytes = m_message.readBytes(s_counterBytes,100)) >= 0) {
        length += numBytes;
      }
      bytes = new byte[length];
      m_message.reset();
      m_message.readBytes(bytes);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
    return bytes;
  }


  public Time getTimeStamp() throws GXO_Exception {
    try {
      return new TimeImpl(m_message.getJMSTimestamp());
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public void setProperty(String name, Object property) throws com.anabas.gxo.GXO_Exception {
    try {
      m_message.setObjectProperty(name,property);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public long getPriority() throws GXO_Exception {
    try {
      return m_message.getJMSPriority();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public long getBodyType() {
    return GMS_Message.BODYTYPE_BYTES;
  }

  public GMS_MessageID getID() {
    return m_id;
  }

  public Enumeration getPropertyNames() throws com.anabas.gxo.GXO_Exception {
    try {
      return m_message.getPropertyNames();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public void clearBody() throws com.anabas.gxo.GXO_Exception {
    try {
      m_message.clearBody();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }
  public long getMessageType() throws GXO_Exception {
    try {
      return m_message.getLongProperty("gxo_message_type");
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public Object getProperty(String name) throws com.anabas.gxo.GXO_Exception {
    try {
      return m_message.getStringProperty(name);
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public void clearProperties() throws com.anabas.gxo.GXO_Exception {
    try {
      m_message.clearProperties();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }
}