package com.anabas.ibus;

import com.anabas.gxo.*;
import com.anabas.concepts.Time;
import com.anabas.util.misc.LogManager;

import ch.softwired.jms.*;
import javax.jms.*;

import java.util.Enumeration;
import java.io.ObjectInput;
import java.io.ObjectOutput;


/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author
 * @version 1.0
 *
 * Wraps an IBus message and present it as an Anabas Event Bus Message.
 *
 * Limitations Currently:
 * (1) Only support string properties.
 *
 * Utilizes following JMS Message Properties:
 * "gxo_message_type" - The message type:  Message.MSGTYPE_MINOR, ...
 *    The value is one of integer value of the definition.
 */

public class GMS_MessageIBusWrapper extends GMS_Message {

  private Message m_message;

  public GMS_MessageIBusWrapper(Message msg) {
    m_message = msg;
  }

  /**
   * Supports only string properties.
   */
  public Object getProperty(String name) {
    String str = "";
    try {
      str = m_message.getStringProperty(name);
    } catch (JMSException e) {
      LogManager.err("Subscriber","Unable to retrieve property",e);
    }
    return str;
  }

  /**
   * @return The JMS Time Stamp header wrapped in a Simple time implementation.
   */
  public Time getTimeStamp() {
    Time t = null;

    try {
      t = new TimeImpl(m_message.getJMSTimestamp());
    } catch (JMSException e) {
      LogManager.err("Subscriber","Unable to get time stamp",e);
    }
    if (t == null) return new TimeImpl(-1);
    return t;
  }

  public void setProperty(String name, Object property) throws GXO_Exception {
    if (!(property instanceof String)) {
      throw new GXO_UnsupportedPropertyTypeException();
    }

    try {
      m_message.setStringProperty(name,(String)property);
    } catch (JMSException e) {
      LogManager.err("Message","Unable to set property " + name,e);
    }
  }

  public long getPriority() {
    long p = -1;
    try {
      p = m_message.getJMSPriority();
    } catch (JMSException e) {
      LogManager.err("Message","Unable to get JMS Priority header",e);
    }
    return p;
  }

  /**
   * @return Detects the ibus message type and return the appropriate message
   *         type tag.  -1 if unknown.
   */
  public long getBodyType() {
    if (m_message instanceof ch.softwired.jms.IBusTextMessage) {
      return GMS_Message.BODYTYPE_TEXT;
    } else if (m_message instanceof ch.softwired.jms.IBusBytesMessage) {
      return GMS_Message.BODYTYPE_BYTES;
    } else if (m_message instanceof ch.softwired.jms.IBusObjectMessage) {
      return GMS_Message.BODYTYPE_OBJECT;
    } else if (m_message instanceof ch.softwired.jms.IBusMapMessage) {
      return GMS_Message.BODYTYPE_MAP;
    }
    // Doesn't work with the applet msrvAppletRT.jar.
    //else if (m_message instanceof ch.softwired.jms.IBusXmlMessage) {
    //return GMS_Message.BODYTYPE_XML;
    //}

    LogManager.err("Message","Found an unknown message type "+m_message.getClass().getName());
    return -1;  // Unknown.
  }

  public GMS_MessageID getID() {
    GMS_MessageID id = null;

    try {
      id = new GMS_MessageIDImpl(m_message.getJMSMessageID());
    } catch (JMSException e) {
      LogManager.err("Message","Unable to extract message id",e);
      id = new GMS_MessageIDImpl("<none>");
    }
    return id;
  }

  /**
   * @return The integer value set in gxo_message_type.  This should really be
   *         a JMS message header but since we can't modify it.
   */
  public long getMessageType() {
    String type = (String)getProperty("gxo_message_type");

    return Integer.parseInt(type);
  }

  public void clearProperties() throws GXO_Exception {
    try {
      m_message.clearProperties();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public void clearBody() throws GXO_Exception {
    try {
      m_message.clearBody();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }

  public Enumeration getPropertyNames() throws GXO_Exception {
    try {
      return m_message.getPropertyNames();
    } catch (JMSException e) {
      throw new GXO_JMSException(e);
    }
  }
}
