package com.anabas.ibus;

import com.anabas.gxo.*;
import com.anabas.concepts.*;
import com.anabas.util.misc.LogManager;

import ch.softwired.jms.*;
import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author
 * @version 1.0
 */

public class GMS_StreamImpl implements GMS_Stream {

  private String m_parentName;    // Parent Context absolute Name.
  private String m_name;          // My name.
  private String m_absoluteName;  // parentName + m_name
  private TopicSession m_session;
  private GMS_MessageSelector m_defaultSelector;

  public GMS_StreamImpl(String parentCtxName,String myName,
                        TopicSession session) {
    m_parentName = parentCtxName;
    m_name       = myName;
    m_session    = session;
    m_defaultSelector = IBusContext.s_defaultSelector;

    if (m_parentName.endsWith("/")) {
      m_absoluteName = m_parentName + m_name;
    } else {
      m_absoluteName = m_parentName + '/' + m_name;
    }
  }

  protected GMS_MessageSelector getDefaultMessageSelector() {
    return m_defaultSelector;
  }

  /**
   * Same as StreamContext.search() but only does so in this stream.
   *
   * @return A destination object that matches the destination selector.
   *         null if no matches.
   */
  public GMS_Destination   findDestination(GMS_DestinationSelector d)
    throws GXO_AccessDeniedException {
    // Really only MulticastDestination so it is also a GMS_Destination
    return (GMS_Destination)d;
  }

  /**
   * @return The name of the stream.
   */
  public String     getName() {
    return m_name;
  }

  /**
   * Utility function to create a TopicSubscriber with all of the appropriate
   * default message selectors.
   */
  protected TopicSubscriber   createIBusSubscriber() throws JMSException {
    Topic topic = IBusJMSContext.getTopic(m_absoluteName);
    return m_session.createSubscriber(topic,m_defaultSelector.toString(),true);
  }

  /**
   * @return null if it can't create a subscriber
   */
  public GMS_StreamSubscriber createSubscriber() {
    try {
      TopicSubscriber sub = createIBusSubscriber();
      GMS_SubscriberImpl subscriber = new GMS_SubscriberImpl(sub,m_session,m_defaultSelector);

      IBusJMSContext.getTopicConnection().start();

      LogManager.log(LogManager.ALL,"Stream","Creating subscriber on '" +
                m_absoluteName + "' with : "+m_defaultSelector+" query string");
      return subscriber;
    } catch (JMSException e) {
      LogManager.err("Stream","Can not create subscriber",e);
      return null;
    }

  }

  /**
   * @return null if it can't create a publisher
   */
  public GMS_StreamPublisher  createPublisher() {
    try {
      Topic topic = IBusJMSContext.getTopic(m_absoluteName);
      GMS_PublisherImpl publisher =
        new GMS_PublisherImpl(m_session.createPublisher(topic),m_session,this);

      LogManager.log(LogManager.ALL,"Stream","Created Publisher on " + m_absoluteName);
      return publisher;
    } catch (JMSException e) {
      LogManager.err("Stream","Can not create publisher",e);
      return null;
    }

  }
}
