/*
 * AudioTalk.java
 *
 * Created on December 18, 2000, 12:30 PM
 */
package com.anabas.commsharedlet;

import audiotalk.hearmevoicelib.*;
//import com.gensym.com.*;
import java.awt.Dimension;
import com.anabas.naming.Context;
import com.anabas.naming.ContextManager;
import com.anabas.util.misc.LogManager;
import com.anabas.sharedlet.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import com.anabas.util.ui.VolumeIndicator;
import com.anabas.util.io.StreamUtil;
import java.io.IOException;
import java.util.Vector;

import com.anabas.gxo.*;
import com.anabas.concepts.*;
import com.anabas.naming.*;


public class AudioTalkInApplication extends javax.swing.JPanel implements Runnable, CapabilityListener,
                                                             AudioControlListener{

  static double MIC_RATIO = 2.5;
  static double SPEAKER_RATIO = 2.5;
  private AudioControl m_audioControl;
  private javax.swing.JSlider m_microphoneSlider, m_speakerSlider;
  private VolumeIndicator m_microphoneIndicator, m_speakerIndicator;
  private JToggleButton m_broadcast;
  private JCheckBox m_handsfree;

  private Vector m_audioListener = new Vector();

  private int status;
  private boolean m_micIsOn = false;
  static final int s_threshold = 1;

  private Thread m_thread;

  private CapabilitiesManager m_capabilitiesManager;
  private User m_myUser;
  private GMS_StreamPublisher m_publisherForPrivate;

  private Color m_selected, m_background;
  public AudioTalkInApplication() {
    initialManagers();
    initComponents();

    this.addAudioControlListener(this);
    m_thread = new Thread(this);
    LogManager.log(LogManager.PRODUCTION,"Communication","auditalk in application start thread");
    m_thread.start();
  }

  private void initialManagers(){
    try {
      Context ctx = ContextManager.getInitialContext();
      m_capabilitiesManager = (CapabilitiesManager)ctx.lookup("services/CapabilitiesManager");
      m_capabilitiesManager.addCapabilityListener(this);

      CommunicationService commService = (CommunicationService)ctx.lookup("services/CommunicationService");
      GMS_Stream streamForPrivate = commService.findStream("application/x-sharedlet-auditorium/private");
      if (streamForPrivate == null) {  // Create the stream if it doesn't exist.
        streamForPrivate = commService.createStream("application/x-sharedlet-auditorium/private");
      }
      m_publisherForPrivate = streamForPrivate.createPublisher();

      SessionManager sessionManager = (SessionManager)ctx.lookup("services/SessionManager");
      m_myUser = sessionManager.getMyUser();

      m_capabilitiesManager = (CapabilitiesManager)ctx.lookup("services/CapabilitiesManager");
      m_capabilitiesManager.addCapabilityListener(this);

      CommunicationControlView controlView = null;
      try{
        controlView = (CommunicationControlView)ctx.lookup("sharedlets/application/x-sharedlet-communication/views/com.anabas.commsharedlet.CommunicationControlView");
      } catch (NamingException e) {
        LogManager.err("Communication audiotlak","Unable to get control view",e);
      }
      if(controlView != null){
        m_audioControl = controlView.getAudioControl();
        this.addAudioControlListener(controlView);
        controlView.addAudioControlListener(this);
        m_audioControl.setAxUserInfo(m_myUser.getUserID().getID());
      }
    } catch (Exception e) {
      LogManager.err("AuditoriumSharedlet","Unable to get comm service",e);
    }
  }

  public void addAudioControlListener(AudioControlListener al){
    m_audioListener.addElement(al);
  }

  private void initComponents() {//GEN-BEGIN:initComponents
    this.setLayout(new GridBagLayout());
    this.setPreferredSize(new Dimension(150,150));
    this.setMinimumSize(new Dimension(150,150));
    m_selected = this.getBackground().darker();
    m_background = this.getBackground();
    m_microphoneSlider = new JSlider();
    m_microphoneIndicator = new VolumeIndicator(6);
    m_speakerSlider = new JSlider();
    m_speakerIndicator = new VolumeIndicator(6);
    m_broadcast = new JToggleButton("Push to Talk");
    m_broadcast.setName("broadcast");
    m_broadcast.setFocusPainted(false);
    m_handsfree = new JCheckBox();
    m_handsfree.setSelected(false);
    m_broadcast.setPreferredSize(new Dimension(100,20));
    m_broadcast.setMargin(new Insets(2,2,2,2));
    m_broadcast.setMinimumSize(m_broadcast.getPreferredSize());
    m_handsfree.setPreferredSize(new Dimension(100,20));
    m_handsfree.setMinimumSize(m_handsfree.getPreferredSize());
    m_handsfree.setFocusPainted(false);
    m_microphoneSlider.setOrientation(javax.swing.SwingConstants.VERTICAL);
    m_microphoneSlider.setMaximum(15);
    m_microphoneSlider.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(javax.swing.event.ChangeEvent evt) {
        SliderHandler(evt);
        microphoneStateChanged(evt);
      }
    }
    );
    m_microphoneSlider.setToolTipText("Microphone Volume");

    m_microphoneIndicator.setMaximum(15);
    m_microphoneIndicator.setBarPreferredSize(new Dimension(16,7));
    m_microphoneIndicator.setEmptyColor(this.getBackground().brighter());
    m_speakerSlider.setOrientation(javax.swing.SwingConstants.VERTICAL);
    m_speakerSlider.setMaximum(15);
    m_speakerSlider.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(javax.swing.event.ChangeEvent evt) {
        speakerStateChanged(evt);
      }
    });
    m_speakerSlider.setToolTipText("Speaker Volume");

    m_speakerIndicator.setMaximum(15);
    m_speakerIndicator.setBarPreferredSize(new Dimension(16,7));
    m_speakerIndicator.setEmptyColor(this.getBackground().brighter());

    m_broadcast.setToolTipText("Hold it to Talk");
    m_broadcast.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mousePressed(java.awt.event.MouseEvent evt) {
        m_audioControl.startTalking();
        fireHoldButtonPressed(true);
      }
      public void mouseReleased(java.awt.event.MouseEvent evt) {
        m_audioControl.stopTalking();
        fireHoldButtonPressed(false);
      }
    });

    m_handsfree.setText("Keep talking");
    m_handsfree.setToolTipText("Select to Talk");
    m_handsfree.setSelected(false);
    m_handsfree.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        if(m_handsfree.isSelected()){
          m_audioControl.startTalking();
          fireHandsFreeSelected(true);
          m_broadcast.setSelected(true);
        }
        else{
          m_audioControl.stopTalking();
          fireHandsFreeSelected(false);
          m_broadcast.setSelected(false);
        }
      }
    }
    );

    m_microphoneSlider.setPreferredSize(new Dimension(16, 80));
    m_microphoneSlider.setMinimumSize(m_microphoneSlider.getPreferredSize());
    m_microphoneIndicator.setPreferredSize(new Dimension(18, 68));
    m_microphoneIndicator.setMinimumSize(m_microphoneIndicator.getPreferredSize());
    m_speakerSlider.setPreferredSize(new Dimension(16, 80));
    m_speakerSlider.setMinimumSize(m_speakerSlider.getPreferredSize());
    m_speakerIndicator.setPreferredSize(new java.awt.Dimension(18, 68));
    m_speakerIndicator.setMinimumSize(m_speakerIndicator.getPreferredSize());

    ImageIcon micIcon = null;
    ImageIcon speakerIcon = null;
    try{
      micIcon = new ImageIcon(StreamUtil.inputStreamToBytes(AudioTalkInApplication.class.getResourceAsStream("mic.gif")));
      speakerIcon = new ImageIcon(StreamUtil.inputStreamToBytes(AudioTalkInApplication.class.getResourceAsStream("speaker.gif")));
    }catch(IOException ie){
      LogManager.err("Audio talk","can not load in image file");
    }

    JLabel micLabel = new JLabel(micIcon);
    micLabel.setPreferredSize(new Dimension(micIcon.getIconWidth(), micIcon.getIconHeight()));
    JLabel speakerLabel = new JLabel(speakerIcon);
    speakerLabel.setPreferredSize(new Dimension(speakerIcon.getIconWidth(),speakerIcon.getIconHeight()));

    constrain(this, speakerLabel,0,0,1,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,0,5,0,0);
    constrain(this, micLabel,2,0,1,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,0,25,0,0);
    constrain(this, m_speakerSlider,0,1,1,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,0,5,0,0);
    constrain(this, m_speakerIndicator,1,1,1,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,6,2,0,0);
    constrain(this, m_microphoneSlider,2,1,1,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,0,25,0,0);
    constrain(this, m_microphoneIndicator,3,1,1,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,6,2,0,0);

    constrain(this, m_broadcast,0,2,4,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,2,5,0,0);
    constrain(this, m_handsfree,0,3,4,1,GridBagConstraints.NONE,
             GridBagConstraints.NORTHWEST,0.0,0.0,2,5,0,0);


    if(m_capabilitiesManager.isCapable(CommunicationSharedletInfo.g_sharedletMIME, "talk")){
      m_microphoneSlider.setEnabled(true);
      m_broadcast.setEnabled(true);
      m_handsfree.setEnabled(true);
    }
    else{
      m_microphoneSlider.setEnabled(false);
      m_broadcast.setEnabled(false);
      m_handsfree.setEnabled(false);
    }

  }//GEN-END:initComponents

  private void speakerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_speakerLeverStateChanged
    m_speakerIndicator.setValue(m_speakerSlider.getValue());
    m_audioControl.setAxSpeakerVolume(m_speakerSlider.getValue());
    fireSpeakerVolumeChanged(m_speakerSlider.getValue());
  }//GEN-LAST:event_speakerLeverStateChanged

  public void fireSpeakerVolumeChanged(int volume){
    for(int i = 0; i< m_audioListener.size(); i++){
      ((AudioControlListener)m_audioListener.elementAt(i)).speakerVolumeChanged(m_speakerSlider.getValue());
    }
  }

  public void speakerVolumeChanged(int volume){
    if(volume != m_speakerSlider.getValue()){
      m_speakerSlider.setValue(volume);
    }
  }

  public void fireHoldButtonPressed(boolean pressed){
    for(int i = 0; i < m_audioListener.size(); i++){
      ((AudioControlListener)m_audioListener.elementAt(i)).holdButtonPressed(pressed);
    }
  }

  public void holdButtonPressed(boolean pressed){
    if(pressed) m_broadcast.setSelected(true);
    else{
      if(!m_handsfree.isSelected()) m_broadcast.setSelected(false);
      else m_broadcast.setSelected(true);
    }
  }

  public void fireHandsFreeSelected(boolean selected){
    for(int i = 0; i < m_audioListener.size(); i++){
      ((AudioControlListener)m_audioListener.elementAt(i)).handsFreeSelected(selected);
    }
  }

  public void handsFreeSelected(boolean selected){
    if(selected && m_handsfree.isSelected()) return;
    if(!selected && !m_handsfree.isSelected()) return;
    if(selected){
      m_handsfree.setSelected(true);
      m_broadcast.setSelected(true);
    }
    else{
      m_handsfree.setSelected(false);
      m_broadcast.setSelected(false);
    }
  }


  private void microphoneStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_micLeverStateChanged
    m_microphoneIndicator.setValue(m_microphoneSlider.getValue());
    m_audioControl.setAxMicVolume(m_microphoneSlider.getValue());
  }//GEN-LAST:event_micLeverStateChanged

  private void SliderHandler(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_SliderHandler

  }//GEN-LAST:event_SliderHandler


  private void constrain( Container container, Component component,
             int grid_x,int grid_y, int grid_width, int grid_height,
             int fill, int anchor, double weight_x, double weight_y,
             int top, int left, int bottom, int right)
   {
      GridBagConstraints c= new GridBagConstraints();
      c.gridx=grid_x; c.gridy=grid_y;
      c.gridwidth=grid_width; c.gridheight=grid_height;
      c.fill=fill; c.anchor=anchor;
      c.weightx=weight_x; c.weighty=weight_y;
      if(top+bottom+left+right>0)
      c.insets = new Insets(top,left,bottom,right);
      ((GridBagLayout)container.getLayout()).setConstraints(component, c);
      container.add(component);
   }



  class MyListener implements _DAXAudioTalkEventsListener{
    public void TunneledMessageIndication(_DAXAudioTalkEventsEvent e0){
//      System.out.println("TunneledMessageIndication "+e0.getArguments());
    }
    public void QueParticipantsChanged(_DAXAudioTalkEventsEvent e0){
//      System.out.println("QueParticipantsChanged "+e0.getArguments());
    }
    public void PositionInQueHasChanged(_DAXAudioTalkEventsEvent e0){
//      System.out.println("PositionInQueHasChanged "+e0.getArguments());
    }
    public void ParticipantsChanged(_DAXAudioTalkEventsEvent e0){
//      System.out.println("ParticipantsChanged "+e0.getArguments());
    }
    public void ParticipantSpeakingIndication(_DAXAudioTalkEventsEvent e0){
//      System.out.println("ParticipantSpeakingIndication "+e0.getArguments());
    }
    public void ParticipantIsMuted(_DAXAudioTalkEventsEvent e0){
//      System.out.println("ParticipantIsMuted "+e0.getArguments());
    }
    public void FloorRequested(_DAXAudioTalkEventsEvent e0){
//      System.out.println("FloorRequested "+e0.getArguments());
    }
    public void ConnectionStatusChanged(_DAXAudioTalkEventsEvent e0){
//      System.out.println("ConnectionStatusChanged "+e0.getArguments());
    }
    public void ConferenceMessageReceived(_DAXAudioTalkEventsEvent e0){
//      System.out.println("ConferenceMessageReceived "+e0.getArguments());
    }
  }

  public void run() {
    int newValue, oldValue=-1;
    while(m_thread != null){
//      int energyValue;
//      if(m_client.getAxMicEnergy() > 0) energyValue = (int)(20.*Math.log((double)(m_client.getAxMicEnergy())));
//      else energyValue = 0;
// System.out.println(".....value: "+energyValue);
//      m_microphoneIndicator.setValue((int)(energyValue*m_microphoneSlider.getValue()/m_microphoneSlider.getMaximum()));

//      if(m_client.getAxSpeakerEnergy() > 0) energyValue = (int)(20.*Math.log((double)(m_client.getAxSpeakerEnergy())));
//      else energyValue = 0;
// System.out.println(".....speaker value: "+energyValue);
//      m_microphoneIndicator.setValue((int)(energyValue*m_microphoneSlider.getValue()/m_microphoneSlider.getMaximum()));

//System.out.println("....energy: "+m_client.getAxSpeakerEnergy()+" "+(int)(SPEAKER_RATIO*m_client.getAxSpeakerEnergy()*m_speakerSlider.getValue()/m_speakerSlider.getMaximum()));
      m_microphoneIndicator.setValue((int)(MIC_RATIO*m_audioControl.getAxMicEnergy()*m_microphoneSlider.getValue()/m_microphoneSlider.getMaximum()));
      m_speakerIndicator.setValue((int)(SPEAKER_RATIO*m_audioControl.getAxSpeakerEnergy()*m_speakerSlider.getValue()/m_speakerSlider.getMaximum()));
      m_microphoneIndicator.repaint(10);
      m_speakerIndicator.repaint(10);
      if(m_microphoneIndicator.getValue() > s_threshold){
        if(!m_micIsOn){
          m_micIsOn = true;
          broadcastMessage(m_myUser,"speak","t");
        }
      }
      else{
        if(m_micIsOn){
          m_micIsOn = false;
          broadcastMessage(m_myUser,"speak","f");
        }
      }

      try{
        m_thread.sleep(300);
      }catch(Exception e){
        LogManager.err("AudioTalkInApplication","thread error", e);
      }
      newValue = m_audioControl.getAxConnectionStatus();
      if(newValue != oldValue){
        setStatus(newValue);
      }
      oldValue = newValue;
    }
  }

  public void setStatus (int n){
//    System.out.println("in status changed: "+n);
    if(n==0){
      this.setToolTipText("not connected");
    }else if(n == 1){
      this.setToolTipText("connecting");
    }else if(n == 2){
      this.setToolTipText("ringing");
    }else if(n == 3){
      this.setToolTipText("connected");
    }else if(n == 4){
       this.setToolTipText("disconnected");
    }
  }

  public String getSharedletMIME(){
    return CommunicationSharedletInfo.g_sharedletMIME;
  }

  public void modeChanged(short newMode) {
  }

  public void roleChanged(String newRole){}

  public void remoteCapabilityChanged(String capability, boolean enabled, UserID userid) {
  }

  public void capabilityChanged(String capability, boolean enabled){
    if(capability.equals("talk")){
      if(enabled == false){
        m_handsfree.setSelected(false);
        m_audioControl.stopTalking();
      }
      m_handsfree.setEnabled(enabled);
      m_broadcast.setEnabled(enabled);
      m_microphoneSlider.setEnabled(enabled);
      m_microphoneIndicator.setEnabled(enabled);
    }
  }

  public void broadcastMessage(User user, String messageType, String messageText){
    try {
      GMS_ObjectMessage msg = (GMS_ObjectMessage)m_publisherForPrivate.createMessage(
               GMS_Message.MSGTYPE_ADMINISTRATION,GMS_Message.BODYTYPE_OBJECT);
      msg.setObject(user);
      msg.setProperty("messagetype", messageType);
      msg.setProperty("messagecontent", messageText);
      m_publisherForPrivate.broadcast(msg);
    } catch (GXO_Exception ex) {
        LogManager.err("auditorium","Unable to broadcast",ex);
    }
  }

}
