package com.anabas.testsharedlet;

/****************************************************************************
 * Used to easily access InfoQuery <item> type parameters.
 * @author David Scott
 ****************************************************************************/
public class IQItem
{
 private String name = "";    //Username (1st part of the JID)
 private String server = "";  //server user is based

 /**
  * none, from - they've subscribe to us, to - we've subscribed to them,
  * both - to + from
  */
 private String subscription = "";
 private String type = ""; //subscription type (subscribe, unsubscribe,,,:

 /**
  * A user may belong to one (* or more *) groups.
  * note: currently only one group per user is supported.
  */
 private String group = "";

 private String nick = "";

 /**
  * May be one/none of the following:
  * subscribe - a request for a subscription is pending
  * unsubscribe - a request for an unsubscribe is pending
  */
 private String ask = "";

 /**
  * Create an InfoQuery Item given the JIDs name, server and type of item.
  * @param sName JID name
  * @param sServer JID server
  * @param sType type of Infoquery item
  */
 public IQItem(String sName, String sServer, String sType )
 {
  this.name = sName;
  this.server = sServer;
  this.type = sType;
  this.nick = sName;
 }

 /**
  * Create an infoquery item given the name, server, nick group
  * @param sName JID name
  * @param sServer JID server
  * @param sNick JID nickname
  * @param sGroup roster group to place user
  */
 public IQItem(String sName, String sServer, String sNick, String sGroup)
 {
  // -------------------------------------
  // Extract jid username
  // -------------------------------------
  this.name = sName;
  this.server = sServer;
  this.nick = sNick;
  this.group = sGroup;
 }

 /**
  * Create an InfoQuery Item given a string containing misc info.
  * Most of the work is done in the constructor.
  * Some or all of the subitems may be included in the string to be parsed.
  * @param sItem String describing the item
  */
 public IQItem(String sItem)
 {
  MiniXMLParser oParser = new MiniXMLParser();

  // -------------------------------------
  // Extract jid username and server name
  // -------------------------------------
  String sJID = oParser.parseTag(sItem, "jid");
  int j = sJID.indexOf("@");		// end of name
  if (j > -1)
  {
   this.name = sJID.substring(0,j);	// get name
   this.server = sJID.substring(j+1);	// get server
  }
  else
  {
   j = sJID.indexOf("/");		// agent item?
   if (j > -1)
   {
    this.server = sJID.substring(0,j);
    return;
   }
  }

  // -------------------------------------
  // Extract subscription
  // -------------------------------------
  this.subscription = oParser.parseTag(sItem, "subscription");

  // -------------------------------------
  // Extract ask
  // -------------------------------------
  this.ask = oParser.parseTag(sItem, "ask");

  // -------------------------------------
  // Extract nickname
  // -------------------------------------
  this.nick = oParser.parseTag(sItem, "name");

  // -------------------------------------
  // Extract jid group (default to friends)
  // -------------------------------------
  this.group = oParser.parseTag(sItem, "group");

  // If no group tag found, default to friends
  if (this.group.length() == 0)
    this.group = "Friends";
 }

 public String getName()
 {
  return(name);
 }

 public String getServer()
 {
  return (server);
 }

 public String getNick()
 {
  return (nick);
 }

 /**
  * Get subscription associated with user
  * @return Subscription type (to, from, both)
  */
 public String getSubscription()
 {
  return (subscription);
 }

 public String getGroup()
 {
  return (group);
 }

 /**
  * Get type associated with user
  * @return type
  */
 public String getType()
 {
  return (type);
 }

 /**
  * Get ask state associated with user
  * @return ask
  */
 public String getAsk()
 {
  return (ask);
 }

 public void setNick( String str )
 {
  nick = str;
 }

 public void setName( String str )
 {
  name = str;
 }

 public void setGroup( String str )
 {
  group = str;
 }
}




