package com.anabas.testsharedlet;

/**
 * Title:        JMSModule.java
 * Description:  JMSModule class.
 *               Converting the HHMP message into JMS message to communicate
 *               with other sharedlet application.
 * Copyright:    Copyright (c) 2001
 * Company:      CSIT FSU
 * @author       Sangyoon Oh soh@csit.fsu.edu
 * @version 1.0
 */

import java.util.*;
import javax.jms.*;

public class JMSModule implements javax.jms.MessageListener
{

    /** Create JMS client for publishing and subscribing to messages. */
    private void setup( String broker, String username, String password)
    {
        // Create a connection.
        try
        {
            javax.jms.TopicConnectionFactory factory;
            factory = (new progress.message.jclient.TopicConnectionFactory (broker));
            connect = factory.createTopicConnection (username, password);
            pubSession = connect.createTopicSession(false,javax.jms.Session.AUTO_ACKNOWLEDGE);
            subSession = connect.createTopicSession(false,javax.jms.Session.AUTO_ACKNOWLEDGE);
        }
        catch (javax.jms.JMSException jmse)
        {
            System.err.println("error: Cannot connect to Broker - " + broker);
            jmse.printStackTrace();
            System.exit(1);
        }

        // Create Publisher and Subscriber to 'chat' topics
        try
        {
            javax.jms.Topic topic = pubSession.createTopic (APP_TOPIC);
            javax.jms.TopicSubscriber subscriber = subSession.createSubscriber(topic);
            subscriber.setMessageListener(this);
            publisher = pubSession.createPublisher(topic);
            // Now that setup is complete, start the Connection
            connect.start();
        }
        catch (javax.jms.JMSException jmse)
        {
            jmse.printStackTrace();
        }

    }


    public void sendJMS(Hashtable hash)
    {
       try
        {
            String s = (String)hash.get("edata");
            if ( s == null )
                System.exit(1);
            else if ( s.length() > 0 )
            {
              javax.jms.TextMessage msg = pubSession.createTextMessage();
              msg.setText( DEFAULT_USERNAME + ": " + s );
              publisher.publish( msg );
            }
        }
        catch ( javax.jms.JMSException jmse )
        {
            jmse.printStackTrace();
        }
    }


    /**
     * Handle the message
     * (as specified in the javax.jms.MessageListener interface).
     */
    public void onMessage( javax.jms.Message aMessage)
    {
        try
        {
            // Cast the message as a text message.
            javax.jms.TextMessage textMessage = (javax.jms.TextMessage) aMessage;

            // This handler reads a single String from the
            // message and prints it to the standard output.
            try
            {
                String string = textMessage.getText();
                Hashtable hs = new Hashtable();
                hs.put("mid", new String("0"));
                hs.put("aid", new String("0"));
                hs.put("tid", new String("0"));
                hs.put("eid", new String("00"));
                hs.put("edata", string);
                parent.setHHMP(hs);
            }
            catch (javax.jms.JMSException jmse)
            {
                jmse.printStackTrace();
            }
        }
        catch (java.lang.RuntimeException rte)
        {
            rte.printStackTrace();
        }
    }


    public JMSModule(PAdaptor pa) {
      String broker    = DEFAULT_BROKER_NAME;
      String username  = DEFAULT_USERNAME;
      String password  = DEFAULT_PASSWORD;
      this.parent = pa;
      setup(broker, username, password);
    }

    private PAdaptor parent;
    private static final String APP_TOPIC = "jms.samples.chat";
    private static final String DEFAULT_BROKER_NAME = "localhost:2506";
    private String DEFAULT_PASSWORD = "password";
    private String DEFAULT_USERNAME = "soh";

    private javax.jms.TopicConnection connect  = null;
    private javax.jms.TopicSession pubSession  = null;
    private javax.jms.TopicSession subSession  = null;
    private javax.jms.TopicPublisher publisher = null;
 }