package com.anabas.sdsharedlet;

import com.anabas.sharedlet.*;
import com.anabas.util.nativeutils.NativeUtilsWrapper;
import com.anabas.util.misc.LogManager;
import java.awt.Dimension;
import java.util.*;

/**
 * Title:        Collaboration Framework
 * Description:  This project contains the Anabas, Inc. collaboration framework libraries.
 * Copyright:    Copyright (c) 2000
 * Company:      Anabas, Inc.
 * @author       Milind
 * @version 1.0
 */

public class SDSharedletInfo extends SharedletRCUtil implements SharedletInfo {

  public static final String g_sharedletMIME = "application/x-sharedlet-sd";

  /* Capability Names */
  public static final String CAP_INPUTCONTROL = "1:input_control";
  public static final String CAP_ANNOTATION = "annotation";
  public static final String CAP_PRESENTATION = "1:presentation";

  /* Moderatable Element Names */
  public static final String ME_INPUTCONTROL = "Input Control";
  public static final String ME_ANNOTATION = "Annotation";
  public static final String ME_PRESENTATION = "Presentation";

  /* Roles */
  public static final String ROLE_HOST = "Host";
  public static final String ROLE_PARTICIPANT = "Participant";

  public static final String REQUEST_FULL_UPDATE = "<RUF>";
  /**
   * Private Member
   */
  private Vector myViewInfo     = new Vector();
  private Vector m_logicInfos   = new Vector();
  private Vector myCapabilities = new Vector();
  private NativeUtilsWrapper m_nativeUtilsWrapper;

  private static String supportedDataTypes[] = { g_sharedletMIME };

  /**
   * Constructor.  Constructs view infos
   */
  public SDSharedletInfo() {
    m_nativeUtilsWrapper = new NativeUtilsWrapper();

    myViewInfo.addElement(new SDControlViewInfo());
    myViewInfo.addElement(new SDPresentationViewInfo());

    m_logicInfos.addElement(new SDSessionLogicInfo());

    String capabilityString = CAP_INPUTCONTROL + "," + CAP_ANNOTATION + "," +
                              CAP_PRESENTATION;
    // setup the list of raw capabilities.
    setCapabilities(capabilityString);

    // setup the moderatable elements
    setModeratable(ME_INPUTCONTROL,CAP_INPUTCONTROL);
    setModeratable(ME_ANNOTATION,CAP_ANNOTATION);
    //setModeratable(ME_PRESENTATION,CAP_PRESENTATION + "," + CAP_INPUTCONTROL);

    String meString = ME_INPUTCONTROL + "," + ME_ANNOTATION;/* + "," +
                              ME_PRESENTATION;*/

    String viewsStirng =  SDControlViewInfo.g_SDControlViewName + "," +
                          SDPresentationViewInfo.PRESENTATION_VIEW_NAME;

    // setup the roles.
    setRole(ROLE_HOST,capabilityString,viewsStirng, meString);
    setRole(ROLE_PARTICIPANT,"",viewsStirng, meString);
  }

  /**
   * @return "Shared Display Sharedlet"
   */
  public String   getDescription() { return "Shared Display Sharedlet"; }

  /**
   * @return application/x-sharedlet-sd
   */
  public String   getMIMEType()   { return g_sharedletMIME; }

  /**
   * @return null
   */
   public String[] getSupportedDataTypes() { return null; }

  /**
   * @return 1.0
   */
  public double   getVersion() { return 1.0; }

  public Vector getLogicInfos() {
    return m_logicInfos;
  }

  public Vector getViewInfos() {
    return myViewInfo;
  }

  public String[] getSupportedDocumentTypes() {
    String strTypes = m_nativeUtilsWrapper.getAllSupportedFileTypes();
    //log("getSupportedDocumentTypes: types [" + strTypes + "]");
    String[] types = getTypesArray(strTypes);
    /*for(int i=0;i<types.length;i++) {
      System.out.print(types[i] + "  ");
    }*/
    return types;
  }

  // ----------------- Private Methods -----------------
  private String[] getTypesArray(String strTypes) {
    StringTokenizer strtok = new StringTokenizer(strTypes,".");
    final int iCountTypes = strtok.countTokens();
    String[] types = new String[iCountTypes];

    for(int iCount = 0;iCount < iCountTypes;iCount++) {
      types[iCount] = strtok.nextToken();
    }
    return types;
  }

  private void err(String ErrMsg, Exception ex) {
    LogManager.err("SDSharedlet::SDSharedletInfo",ErrMsg,ex);
  }

  private void log(String msg) {
    LogManager.log("SDSharedlet::SDSharedletInfo",msg);
  }
}