package com.anabas.testsharedlet;

/****************************************************************************
 * This class handles only simple Jabber Command XML parsing(meant to be small)
 * @author    Jonathan Paulson
 * @version     $ 0.7.3 $
 ****************************************************************************/
public class MiniJabberParser
{
 private MiniXMLParser moXMLParser = null;
 private IJabberClient moClient = null;
 private boolean       mbLookingForBeginTag = true;
 private String        msXMLDoc = "";
 private String        msTagName = "";
 private int           miBracketNbr = 0;
 private static String MESSAGE = "message";
 private static String PRESENCE = "presence";
 private static String INFO_QUERY = "iq";

 /**
  * Class Constructor
  * @param client  Reference to the Object that implements IJabberClient
  *                Used to communicate back to the object
  */
 public MiniJabberParser(IJabberClient client)
 {
  moClient = client;
  moXMLParser    = new MiniXMLParser();
 }

 /**
  * This is a VERY simple Jabber parser.
  * If the XML tag isn't supported then it's ignored but
  * calls IClient.unsupportedTag() to allow the client to
  * handle if they want to
  * @param sXML       XML to parse
  */
 public synchronized void ProcessJabber(String XML, boolean mbEchoSentXLM)
 {
  if (mbLookingForBeginTag)
  {
   msXMLDoc += XML;
   int iEndBracket = msXMLDoc.indexOf(">");
   //search for the begin tag
   if (iEndBracket != -1)
   {
    mbLookingForBeginTag = false;
    XML = "";

    //look for first space after the tag name so that we
    //know how to get the tag name
    int iEndTag = msXMLDoc.indexOf(" ");
    if (iEndTag == -1)
    {
     //no space, pure tag name (remove Brackets) ie... <Presence>
     msTagName = msXMLDoc.substring(1,iEndBracket-1);
     if (msTagName.endsWith("/"))
     {
      //Begin and End tag in one ie... <Presence/>
      msTagName.substring(0, msTagName.length()-1);
     }
    }
    else
    {
  //tag consists of some elements ie... <Presence to="Savannah" from="Sierra"/>
     //so extract just the tag name
     msTagName = msXMLDoc.substring(1,iEndTag);
    }

    //Check to see if it's one of the tags that we are ignoring, if so ignore
    if (ignoreTag(msTagName))
    {
     mbLookingForBeginTag = true;
     msXMLDoc = "";
     msTagName = "";
    }
   }
  }

  //If the begin tag was found, now lets search for end tag
  if (!mbLookingForBeginTag)
  {
   msXMLDoc += XML;
   boolean bBracketsComplete = false;

   //Keep track of the number of brackets so that we can deduce
   //if we are in the middle of a tag
   if (XML.equals("<") || XML.equals(">"))
   {
    miBracketNbr++;
    bBracketsComplete = (miBracketNbr % 2 == 0);
   }

   //use temp variables that contain the lowercase versions of our variables
   //to increase speed
   String sLowerCaseXMLDoc = msXMLDoc.toLowerCase();
   String sLowerCaseTag = msTagName.toLowerCase();

   //Search for XML end tag which is "tagname>" or just "/>"
   //If it is "/>" then make sure
   //it is ending the the tag we are looking for by only
   //accepting "/>" where there is an odd number of brackets
   if(sLowerCaseXMLDoc.endsWith(msTagName + ">") ||
      (sLowerCaseXMLDoc.endsWith("/>") && !bBracketsComplete))
   {
    if (mbEchoSentXLM)
    {
     System.out.println("RECV: " + msXMLDoc);
    }
    if (sLowerCaseTag.equals(MESSAGE))
    {
     processMessage(msXMLDoc);
    }
    else if (sLowerCaseTag.equals(PRESENCE))
    {
     processPrescence(msXMLDoc);
    }
    else if (sLowerCaseTag.equals(INFO_QUERY))
    {
     processInfoQuery(msXMLDoc);
    }
    else
    {
     //Not supported
     if (moClient != null)
     {
      moClient.unsupportedTag(msTagName, msXMLDoc);
     }
    }
    //We are done processing this tag,
    //reset to orginal state to look for next tag
    mbLookingForBeginTag = true;
    msXMLDoc = "";
    msTagName = "";
    miBracketNbr = 0;
   }
  }
 }

 /**
  * Parse out the Jabber Message elements and call
  * IClient.msgReceived()
  * @param XMLDoc       XML to parse
  */
 private void processMessage(String XMLDoc)
 {
  //We have the whole Message, so now parse apart the tags
  if (moClient != null)
  {
   String from = moXMLParser.parseTag(XMLDoc, "from");
   String sType = moXMLParser.parseTag(XMLDoc, "type");
   String sSubject = moXMLParser.parseTag(XMLDoc, "subject");
   String sBody = moXMLParser.parseTag(XMLDoc, "body");
   String sThread = moXMLParser.parseTag(XMLDoc, "thread");

   //Check to see if it's JabberIM's special X messages
   /*
   String sX = moXMLParser.parseTag(XMLDoc, "x", true);
   if (sX.length() > 0)
   {
    String sXMLns = moXMLParser.parseTag(sX, "xmlns");
    if (sXMLns.indexOf("x") != -1)
    {
     String sComposing = moXMLParser.parseTag(XMLDoc, "composing", true);
     String sReply = moXMLParser.parseTag(sComposing, "reply", true);
     if (sReply.length() > 0)
     {
      moClient.msgReplyStarted(from, sReply);
      return;
     }
    }
   }
   */
   moClient.msgReceived(from, sType, sSubject, sBody, sThread);
  }
 }

 /**
  * Parse out the Jabber Prescence elements and call
  * IClient.presenceReceived()
  * @param XMLDoc       XML to parse
  */
 private void processPrescence(String XMLDoc)
 {
  //We have the whole Presence, so now parse apart the tags
  if (moClient != null)
  {
   String sTo = moXMLParser.parseTag(XMLDoc, "to");
   String sFrom = moXMLParser.parseTag(XMLDoc, "from");
   String sType = moXMLParser.parseTag(XMLDoc, "type");
   String sStatus = moXMLParser.parseTag(XMLDoc, "status");
   String sPriority = moXMLParser.parseTag(XMLDoc, "priority");
   String sShow = moXMLParser.parseTag(XMLDoc, "show");
   String sError = moXMLParser.parseTag(XMLDoc, "error");
   String sCode = moXMLParser.parseTag(XMLDoc, "code");
   moClient.presenceReceived(sTo, sFrom, sType, sStatus, sPriority,
                             sShow, sError, sCode);
  }
 }

 /**
  * Parse out the Jabber InfoQuery elements and call
  * IClient.infoQueryReceived()
  * @param XMLDoc       XML to parse
  */
 private void processInfoQuery(String XMLDoc)
 {
  //We have the whole InfoQuery, so now parse apart the tags
  if (moClient != null)
  {
   String to = moXMLParser.parseTag(XMLDoc, "to");
   String sFrom = moXMLParser.parseTag(XMLDoc, "from");
   String sID = moXMLParser.parseTag(XMLDoc, "id");
   String sType = moXMLParser.parseTag(XMLDoc, "type");
   String sQuery = moXMLParser.parseTag(XMLDoc, "query");
   String sError = moXMLParser.parseTag(XMLDoc, "error");
   String sCode = moXMLParser.parseTag(XMLDoc, "code");
   moClient.infoQueryReceived(to, sFrom, sID, sType, sQuery, sError, sCode);
  }
 }

 /**
  * Check to see this tag should be ignored.
  * @return boolean    True if you should ignore this tag, false otherwise
  */
 private boolean ignoreTag(String tag)
 {
  //These tags always come through and the end tags don't come
  //until the Connection is closed
  //so they can be safely ignored for now
  if (tag.toLowerCase().equals("?xml") ||
      tag.toLowerCase().equals("stream:stream"))
  {
   return true;
  }
  else
  {
   return false;
  }
 }
}




