package com.anabas.sdsharedlet;

import java.awt.event.*;
import java.util.*;
import java.awt.*;

import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import com.anabas.naming.*;
import com.anabas.concepts.*;
import com.anabas.gxo.*;

/**
 * Title:        Shared Display Sharedlet
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Anabas Inc.
 * @author
 * @version 1.0
 */

public class SDPresentationView implements  SharedletView,
                                            GMS_MessageListener
                                            {

  private static final String PROP_REQ_TYPE = "req1";
  static final String JAVA_RENDERER_MIME = "application/java";

  private SDNativeRenderer m_renderer;
  private SDControlView m_controlView;
  private SDWrapper m_sdWrapper;
  private SDEventListenerThread   m_eventListenerThread;

  private boolean   m_host = false;
  private boolean   m_participant = false;
  private String    m_role;
  private int[]     m_appsToRestore = new int[0];
  private boolean   m_startPosting = false;
  private boolean   m_bIsPresenter = false;

  private SharedletManager        m_sharedletManager;
  private SDSharedlet             m_sdsharedlet;
  private SharedletInfo           m_sharedletInfo;
  private SharedletViewManager    m_viewManager;
  private CapabilitiesManager     m_capabilitiesManager;
  private SessionManager          m_sessionManager;
  private CommunicationService    m_commService;
  private GMS_Stream              m_stream;
  private GMS_StreamPublisher     m_publisher;
  private GMS_StreamSubscriber    m_subscriber;
  private User                    m_thisUser;
  private Vector                  m_RFUQueue = new Vector();

  public SDPresentationView() {
    log("Creating Presentation View ...");
    try {
      m_controlView = (SDControlView) getMyView(
                        SDControlViewInfo.g_SDControlViewName);
    }
    catch (Exception ex) {
      err("SDPresentationView: Err:" , ex);
    }

    if(m_controlView == null) {
        log("### SDPresentationView: could not get the SDControl view ");
        m_renderer = new SDNativeRenderer(null);
    } else {
      m_sdWrapper = m_controlView.getSDWrapper();
      m_renderer = new SDNativeRenderer(m_sdWrapper);
    }

    getServices();
  }

  public void activate() {
    log("activate: called ");
    //initialize the native code ...
    m_sdWrapper.init();
    //log("Native code initialized ...");

    // create the thread that will listen to the native events ...
    m_eventListenerThread = new SDEventListenerThread(m_publisher, m_stream,
                                      m_sdWrapper, this,m_controlView);

    testWndListener();

    boolean canControlInput = canControlInput();
    boolean hasAnnotationControl = canAnnotate();
    long presentationToken = m_controlView.getToken(SDSharedletInfo.CAP_PRESENTATION);
    long icToken = m_controlView.getToken(SDSharedletInfo.CAP_INPUTCONTROL);

    log("Got Presentation Token ==========[" + presentationToken + "]===========");
    log("Got input control Token ==========[" + icToken + "]===========");

    m_bIsPresenter = canPresent();
    if(m_bIsPresenter)
    {
log("==========> Presenter >> activated with Input control set to [" + canControlInput + "]");
      m_sdWrapper.listen(presentationToken);
      m_sdWrapper.EnableInputControl(canControlInput,icToken);
      m_sdWrapper.enableAnnotation( hasAnnotationControl );
      // start the event listening thread
      m_eventListenerThread.beginThread(true);
      restoreApps();
      m_controlView.enableRemoveURL(true);
      // get all participants currently logged in ...
      //Vector users = m_sessionManager.getParticipants();
      // add those participants in RFU queue
    } else {
log("==========> Participant >> activated with Input control set to [" + canControlInput + "]");

      m_sdWrapper.connect(presentationToken);
      // start the event listening thread
      m_eventListenerThread.beginThread(false);

      m_sdWrapper.EnableInputControl(canControlInput,icToken);
      m_sdWrapper.enableAnnotation( hasAnnotationControl );

      // broadcast the REQUEST_FULL_UPDATE message so that the host
      // can get this message ...
      UserID id = m_thisUser.getUserID();
      log("<<<<<<< Participant broadcasting the Request Full Update >>>>>");
      log("User id = " + id);
      broadcastObjectMessage(id,PROP_REQ_TYPE,SDSharedletInfo.REQUEST_FULL_UPDATE);
    }
  }

  public void deactivate() {
    log("deactivate: called ...");
    if(m_eventListenerThread != null) {
      m_controlView.enableRemoveURL(false);
      m_appsToRestore = m_controlView.getSharedWnds();
//log("############ got [" + m_appsToRestore.length + "] wnds from control view ");
      m_sdWrapper.release();
      m_eventListenerThread.stopThread();
    }
  }

  private boolean canPresent() {
    return m_capabilitiesManager.isCapable(SDSharedletInfo.g_sharedletMIME,
                         SDSharedletInfo.CAP_PRESENTATION);
  }

  private boolean canControlInput() {
    return m_capabilitiesManager.isCapable(SDSharedletInfo.g_sharedletMIME,
                         SDSharedletInfo.CAP_INPUTCONTROL);
  }

  boolean canAnnotate() {
    return m_capabilitiesManager.isCapable(SDSharedletInfo.g_sharedletMIME,
                         SDSharedletInfo.CAP_ANNOTATION);
  }

  private void restoreApps() {
  log("######### restoring apps [" + m_appsToRestore.length + "] ");
    for(int iCount = 0;iCount < m_appsToRestore.length;iCount++) {
  log("######### restoring app [" + m_appsToRestore[iCount] + "] ");
      if(!m_sdWrapper.ClickToShare(m_appsToRestore[iCount],SDWrapper.CTS_SHARE)) {
        m_sdsharedlet.removeURL(new Integer(m_appsToRestore[iCount]));
      }else {
        log("####### app restored successfully #####");
      }
    }
    int lastActivatedWnd = m_controlView.getLastActivatedWnd();
    if(lastActivatedWnd != 0)
      m_sdWrapper.ClickToShare(lastActivatedWnd,SDWrapper.CTS_ACTIVATE);
  }

  public SharedletViewRenderer getRenderer(String MIMEType) {
    log("getRenderer: called with MIME type [" + MIMEType + "]");
    if (MIMEType.equals(JAVA_RENDERER_MIME)) {
      return m_renderer;
    }
    return null;
  }

  /* ----- GMS_MessageListener Methods ----- */
  public void onMessage(GMS_Message m) {
    int MsgBodyType = (int) m.getBodyType();
    if(m_bIsPresenter) {
      processHostMessage(m, MsgBodyType);
    } else {
      processParticipantMessage(m, MsgBodyType);
    }
  }

  public void onRequest(GMS_Message m,GMS_MessageReplyer r) {
  }

  synchronized void addToRFUQue(UserID userId) {
    m_RFUQueue.addElement(userId);
  }

  synchronized UserID getNextInRFUQue() {
    UserID userId = null;
    try {
      userId = (UserID) m_RFUQueue.elementAt(0);
      m_RFUQueue.removeElement(userId);
    }
    catch (ArrayIndexOutOfBoundsException ex) {
    }
    return userId;
  }

  private void processHostMessage(GMS_Message m,int MsgBodyType) {
    switch(MsgBodyType) {
    case GMS_Message.BODYTYPE_OBJECT:
      try {
        GMS_ObjectMessage msg = (GMS_ObjectMessage) m;
log("!!!!!!!!!!!!! Host got the object message !!!!!!!!!!!!!!");
        String strVal = (String) msg.getProperty(PROP_REQ_TYPE);
        if(strVal.equals(SDSharedletInfo.REQUEST_FULL_UPDATE)) {

          UserID newUserID = (UserID) msg.getObject();
          log("!!!!!!!!! Host sending RFU for user [" + newUserID.getID() + "] !!!!!!!!!!!");
          addToRFUQue(newUserID);
          m_eventListenerThread.requestFullUpdate(SDEventListenerThread.RFU_USER_REQUEST);
        }
      } catch (GXO_Exception e) {
        err("processHostMessage: Error",e);
      }
      break;

    case GMS_Message.BODYTYPE_BYTES:
      try {
        GMS_BytesMessage msg = (GMS_BytesMessage)m;
        byte[] eventBuf = msg.getBytes();

        // post this event to the native code ..
        m_sdWrapper.PostEvent(eventBuf);
      } catch (GXO_Exception e) {
        err("processHostMessage: Error",e);
      }
      break;

    default:
      //log("processHostMessage: got invalid message ...");
      break;
    }
  }

  private void processParticipantMessage(GMS_Message m,int MsgBodyType) {
//log("processParticipantMessage: got some message ...");
    switch(MsgBodyType) {
    case GMS_Message.BODYTYPE_BYTES:
//log("processParticipantMessage: got bytes message ...");
      try {
        GMS_BytesMessage msg = (GMS_BytesMessage)m;
        byte[] eventBuf = msg.getBytes();
//log("participant got msg [" + eventBuf[0] + "]");
        // post this event to the native code ..
        m_sdWrapper.PostEvent(eventBuf);
      } catch (GXO_Exception e) {
        err("processParticipantMessage: Error",e);
      }
      break;

    default:
      // if not bytes message for participant .. ignore it
      log("processParticipantMessage: got invalid message ...");
      return;
    }
  }


  private boolean getServices() {
    try {
      Context ctx = ContextManager.getInitialContext();
      // get a handle to the Communication service ...
      m_commService = (CommunicationService)ctx.lookup("services/CommunicationService");
// 4/24/01, by Xu Geng, using another topic connection
      Hashtable params = new Hashtable();
      params.put("type","streaming");
      m_stream = m_commService.findStream(SDSharedletInfo.g_sharedletMIME, params);
      if (m_stream == null) {  // Create the stream if it doesn't exist.
        m_stream = m_commService.createStream(SDSharedletInfo.g_sharedletMIME, params);
      }
      m_publisher = m_stream.createPublisher();
      m_subscriber = m_stream.createSubscriber();
      m_subscriber.setMessageListener(this);

      // get a handle to the Capabilities Manager
      m_capabilitiesManager = (CapabilitiesManager)ctx.lookup("services/CapabilitiesManager");
      //m_capabilitiesManager.addCapabilityListener(this);

      m_role = m_capabilitiesManager.getRole(SDSharedletInfo.g_sharedletMIME);

      if(m_role == null) {
        log("Could not get role from the Capabilities manager ...");
      } else {
        //log("role set to : " + m_role);
        if(m_role.equals(SDSharedletInfo.ROLE_HOST)) {
          m_host = true;
        }
        else if(m_role.equals(SDSharedletInfo.ROLE_PARTICIPANT))
          m_participant = true;
      }

      m_sharedletManager = (SharedletManager) ctx.lookup("services/SharedletManager");
      m_sdsharedlet = (SDSharedlet) m_sharedletManager.getSharedlet(
                          m_sharedletManager.getInfo(SDSharedletInfo.g_sharedletMIME));

      m_sessionManager = (SessionManager)ctx.lookup("services/SessionManager");

      m_thisUser = m_sessionManager.getMyUser();

      //log("The current user is [" + m_thisUser.getFirstName() + "]");
/*      Vector p = m_sessionManager.getParticipants();
      Enumeration enum = p.elements();
      while(enum.hasMoreElements()) {
        m_userList.addItem(((User)enum.nextElement()).getUserID().getID());
      }*/
    }
    catch (Exception ex) {
        err("getServices: Error",ex);
        return false;
    }
    return true;
  }

  private void broadcastTextMessage(String strMsg,String propName,Object val) {
    try {
      GMS_TextMessage msg = (GMS_TextMessage) m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,
                                    GMS_Message.BODYTYPE_TEXT);
      if(msg == null) {
        log("broadcastTextMessage: null message ...");
        SDSharedlet.handleDisconnected();
        return;
      }

      msg.setText(strMsg);
      if(propName != null) {
        msg.setProperty(propName,val);
      }
      m_publisher.broadcast(msg);
    } catch (Exception ex) {
      SDSharedlet.handleDisconnected();
      err("broadcastTextMessage: Error",ex);
    }
  }

  private void broadcastObjectMessage(Object objMsg,String propName,String propVal) {
    try {
      GMS_ObjectMessage msg = (GMS_ObjectMessage) m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,
                                    GMS_Message.BODYTYPE_OBJECT);
      if(msg == null) {
        log("broadcastObjectMessage: null message ...");
        SDSharedlet.handleDisconnected();
        return;
      }

      msg.setObject((java.io.Serializable)objMsg);

      if(propName != null) {
        msg.setProperty(propName,propVal);
      }
      m_publisher.broadcast(msg);
    } catch (Exception ex) {
      SDSharedlet.handleDisconnected();
      err("broadcastObjectMessage: Error",ex);
    }
  }

  private void testWndListener() {
    // for testing ...
    m_renderer.addComponentListener(new ComponentAdapter() {
      public void componentHidden(ComponentEvent e) {
        log("============> sd wnd hidden ");
      }

      public void componentMoved(ComponentEvent e) {
        Point p = m_renderer.getLocation();
        log("============> sd wnd moved to [" + p.x + "," + p.y + "]");
      }

      public void componentResized(ComponentEvent e) {
        Dimension d = m_renderer.getDimension();
        log("============> sd wnd resized to [" + d.width + "," + d.height + "]");
      }

      public void componentShown(ComponentEvent e) {
        log("============> sd wnd shown ");
      }
    });

    m_renderer.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        log("============> sd wnd closing ");
      }

      public void windowClosed(WindowEvent e) {
        log("============> sd wnd closed ");
      }

      public void windowOpened(WindowEvent e) {
        log("============> sd wnd opened ");
      }

      public void windowIconified(WindowEvent e) {
        log("============> sd wnd iconified ");
      }

      public void windowDeiconified(WindowEvent e) {
        log("============> sd wnd deiconified ");
      }

      public void windowActivated(WindowEvent e) {
        log("============> sd wnd activated ");
      }

      public void windowDeactivated(WindowEvent e) {
        log("============> sd wnd deactivated ");
      }
    });
  }

  void initJMS() {
  }

  void refresh() {
    m_renderer.refresh();
  }

  void setAsPresenter(boolean bPresenter) {
    m_bIsPresenter = bPresenter;
  }

  private boolean getMyViewInfo() throws Exception {
    Context ctx = ContextManager.getInitialContext();
    SharedletManager sharedletManager = (SharedletManager) ctx.lookup("services/SharedletManager");

    m_sharedletInfo = sharedletManager.getInfo(SDSharedletInfo.g_sharedletMIME);

    if(m_sharedletInfo == null) {
      throw new Exception("### getMyViewInfo: cannot get sharedlet info for [" +
                  SDSharedletInfo.g_sharedletMIME + "]");
    }
    m_viewManager = sharedletManager.getViewManager();

    return m_viewManager != null;
  }

  SharedletView getMyView(String viewName) throws Exception {
    if((m_sharedletInfo == null || m_viewManager == null) && !getMyViewInfo())
    {
      log("==========>> 1 : " + m_sharedletInfo + ", " + m_viewManager + "<<======");
      return null;
      }

    Vector views = m_sharedletInfo.getViewInfos();
    if(views == null) {
      throw new Exception("### getMyView: cannot get views for [" + SDSharedletInfo.g_sharedletMIME + "]");
    }

    SharedletViewInfo viewInfo = null;
    Enumeration enum = views.elements();
    while (enum.hasMoreElements()) {
      SharedletViewInfo item = (SharedletViewInfo) enum.nextElement();
      if(item.getID().equals(viewName)) {
        viewInfo = item;
        break;
      }
    }

    if(viewInfo == null) {
      throw new Exception("### getMyView: cannot get view named [" + viewName + "]");
    }
    return m_viewManager.getView(viewInfo);
  }


  SDEventListenerThread getEventListenerThread() {
    return m_eventListenerThread;
  }

  private void err(String ErrMsg, Exception ex) {
    LogManager.err("SDSharedlet::SDPresentationView",ErrMsg,ex);
  }

  private void log(String msg) {
    LogManager.log("SDSharedlet::SDPresentationView",msg);
  }
}