package com.anabas.pdasharedlet;

import com.anabas.concepts.*;
import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import com.anabas.gxo.*;
import com.anabas.naming.*;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Vector;

public class AnabasConnection extends Thread
                              implements GMS_MessageListener, SessionListener
{
 private IAnabasConnectionClient moConnectionClient = null;
 private AnabasConnectionState   moConnState = null;

 private SessionManager       m_sessionManager;
 private CommunicationService m_commService;
 private GMS_StreamPublisher  m_publisher;
 private GMS_StreamSubscriber m_subscriber;

 public AnabasConnection(IAnabasConnectionClient connectionClient)
 {
  // Grab the handle to the communication service and
  // get the handle to the event bus stream for this sharedlet.
  try {
     Context ctx = ContextManager.getInitialContext();
     m_commService = (CommunicationService)ctx.lookup("services/CommunicationService");
     // Sharedlets should use their MIME type or some derivative of it
     // as the name of the stream to avoid namespace collisions.
     // Try to find an existing stream.
     //--GMS_Stream stream = m_commService.findStream(TestSharedletInfo.g_sharedletMIME);
     GMS_Stream stream = m_commService.findStream("application/x-sharedlet-communication/public");

     // If the stream doesn't exist then create it.
     if (stream == null)
     {
      stream = m_commService.createStream(PdaSharedletInfo.g_sharedletMIME);
     }

     // Create a publisher & subscriber to the stream
     // so it can receive and send messages to the stream.
     m_publisher = stream.createPublisher();
     m_subscriber = stream.createSubscriber();

     // Set itself as the message listener waiting for incoming messages and
     // requests.
     m_subscriber.setMessageListener(this);

     // Get a handle to the session manager and act as a listener
     // to update the user list selector.
     m_sessionManager = (SessionManager)ctx.lookup("services/SessionManager");
     m_sessionManager.addSessionListener(this);
  }
  catch (Exception e) {
       LogManager.err("PdaControlView", "Unable to get comm service", e);
  }
 }


 // Do nothing.
 public void activate()
 {
  LogManager.log("View", getID() + " view activated");
 }


 // Do nothing.
 public void deactivate()
 {
  LogManager.log("View", getID() + " view deactivated");
 }


 // @return "application/x-sharedlet-docnav"
 public String getSharedletMIME()
 {
  return PdaSharedletInfo.g_sharedletMIME;
 }

 // Invoked when this sharedlet's role has changed.
 public void roleChanged(String newRole)
 {
  LogManager.log("PdaSharedlet","Role changed to : "+newRole);
 }


 // Listen for remote capability changes.
 public void remoteCapabilityChanged(String capability,boolean enabled,UserID user)
 {
  LogManager.log("PdaControlView", "Remote '" + capability + "' is now " +
                 enabled + " for user: " + user);
 }


 // Listen for capability changes and enable/disable appropriate set of buttons
 // changes.
 public void capabilityChanged(String capability,boolean enabled)
 {
  LogManager.log("PdaControlView", "My '" + capability+"' is now " + enabled);
 }

 // Invoked by SessionManager service when a user joins.
 // Need to update the user list.
 public void userJoined(SessionEvent e)
 {
  User u = (User)e.getSource();
  //m_userList.addItem(u.getUserID());
 }

 // Invoked by SessionManager service when a user leaves.
 // Need to update the user list.
 public void userLeft(SessionEvent e)
 {
  User u = (User)e.getSource();
  //m_userList.removeItem(u.getUserID());
 }


 // When it receives a message from the bus containing a new random number
 // that was generated by another user, display it in the presentation area.
 public void onMessage(GMS_Message message)
 {
  String sResponse = null;

  if (message instanceof GMS_TextMessage)
  {
   GMS_TextMessage msg = (GMS_TextMessage)message;
   try {
      String messageType = (String)msg.getProperty("messagetype");
      if (messageType.equals("textchat"))
      {
       String userName = (String)msg.getProperty("username");
       String userID = (String)msg.getProperty("userid");
       //if (!userID.equals("kangskim"))  //m_myUser.getUserID().getID()))
       //{
         //dataReceived(userName, msg.getText());
         //writeText(userName + ": " + msg.getText());
         //fireNewTextChatEvent();
       //}
       sResponse = msg.getText();

       if (sResponse != null)
         dataReceived(userName, sResponse);
      }
   }
   catch (GXO_Exception e) {
        LogManager.err("Textchat", "Unable to extract the message", e);
   }
  }
 }

// The message type used in broadcast is GMS_TextMessage with 3 properties:
 // "messagetype", "username", "userid"  and textchat message
 public synchronized boolean broadcastMessage(String str)
 {
  StringTokenizer st = new StringTokenizer(str, "\0");

  String messageType = st.nextToken();
  String userName = st.nextToken();
  String userID = st.nextToken();
  String messageText = st.nextToken();
  System.out.println("messageType : " + messageType);
  System.out.println("userName : " + userName);
  System.out.println("userID : " + userID);
  System.out.println("messageText : " + messageText);

  try {
     GMS_TextMessage msg = (GMS_TextMessage)m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR, GMS_Message.BODYTYPE_TEXT);


     msg.setProperty("messagetype", messageType);  // messageType = "textchat"
     msg.setProperty("username", userName);  //m_myUserName);
     msg.setProperty("userid", userID);  //m_myUser.getUserID().getID());
     msg.setText(messageText);
     m_publisher.broadcast(msg);

     return true;
  }
  catch (GXO_Exception ex) {
       LogManager.err("Textchat", "Unable to broadcast", ex);
       return false;
  }
 }

 // These function are not used.
 public void onRequest(GMS_Message m, GMS_MessageReplyer r){}


 // @return The unique name of the control view.
 public String getID()
 {
  return "Pda Control";
 }

 private void dataReceived(String name, String message)
 {
  if (moConnectionClient != null)
  {
   moConnectionClient.dataReceived(name, message);
  }
 }


 private void error(String str)
 {
  if (moConnectionClient != null)
  {
   moConnectionClient.error(str);
  }
 }
}




