#!/bin/sh

# 
# Start the Progress SonicMQ broker
# 

cd "/opt/anabas/sonic"

# JRE/JVM and library paths must be set by setenv
. /opt/anabas/sonic/bin/setenv

# If you want to use Oracle 8i database, set SONICMQ_DBCLASS variable to point to Oracle JDBC driver
if [ "x$SONICMQ_DBCLASS" = "x" ]
then
    SONICMQ_DBCLASS=$SONICMQ_LIB/defdb_server.jar
fi

usage ()
{
    echo "
    Usage: startbr [-f iniFile] [-p password] [-x]
    
    Options:
    
    -f iniFile    The iniFile is the path for the INI file used to configure 
                  the SonicMQ Broker.  (default is broker.ini)

    -p password   An encryption key is derived from this password and used to
                  decrypt the INI file specfied by the -f parameter.  If a -p
                  option is provided, then the INI file is assumed to be 
                  encrypted.  If no password is provided, then the INI file 
                  is assumed to be in clear-text format.

    -x            Indicates that the value passed to the -p option is an encrypted
                  value.
    "
}

# initialize the inifile and inipwd values.
inifile=broker.ini
inipwd=
encflag=

# parse the parameters
while getopts f:p: option
do
    case $option in
	f)  inifile=$OPTARG
	    ;;

	p)  inipwd=$OPTARG
	    ;;

      x)  inipwd=$OPTARG
          encflag="-x"
          ;;

	\?) usage
	    exit 1
	    ;;
    esac
done

echo $$ > /opt/anabas/sonic/sonicMQ_pid

$SONICMQ_JRE $JVM_PARAM -classpath "$JRE_CLASSPATH:$SONICMQ_LIB/broker.jar:$SONICMQ_LIB/jetty.jar:$SONICMQ_LIB/webclient.jar:$SONICMQ_LIB/gnu-regexp-1.0.6.jar:$SONICMQ_DBCLASS:$SONICMQ_SSL_LIB" progress.message.broker.Broker $inifile $inipwd $encflag
