/****************************************************************************
 * This class handles only basic Anabas server communication.
 * @author  Kangseok Kim
 ****************************************************************************/
package com.anabas.pdasharedlet;

import java.util.*;

public class AnabasComm implements IAnabasConnectionClient
{
 private IAnabasClient       moClient       = null;
 private AnabasConnection    anabasConn     = null;
 private boolean             mbEchoXML      = false;
 private static final String STREAM_END     = "</stream:stream>";
 private static final String DEFAULT_HOST   = "sycamore.csit.fsu.edu";
 private static final int    DEFAULT_PORT   = 443;


 // @param client Reference to the Object that implements IAnabasClient
 //               Used to communicate back to the object
 public AnabasComm(IAnabasClient client)
 {
  moClient = client;
 }


 // Set/reset the xml echo/debug flag
 // @param bDebug true = output xml, false = no debug output
 public void setDebug(boolean bDebug)
 {
  mbEchoXML = bDebug;
 }

//================= Required IAnabasConnectionClient methods =================

 // Raw Data received from host
 public void dataReceived(String name, String body)
 {
  moClient.anabasMsgReceived(name, body);
 }


 public void connectionClosed()
 {
  moClient.connectionClosed();
  moClient.rawXML("\nDisconnected");
 }


 public void error(String str)
 {
  moClient.rawXML(str);
  moClient.error(str);
 }
//============================================================================

 public int connect(String host, String hostName, String port)
 {
  String sXML = "";

  //Disconnect if already connected
  if (anabasConn != null)
  {
   //anabasConn.disconnect();
  }
  // Open up communication pipe
  anabasConn = new AnabasConnection(this);
  anabasConn.start();
  /*
  if (anabasConn.connect(host, port).state() == AnabasConnectionState.OPEN)
  {
   // Connect
   return sendRawXML(sXML);
  }
  else
  {
   return Constants.UNABLE_TO_CONNECT;
  }
  */
  return sendRawXML(sXML);
 }


 public int disconnect()
 {
  if (anabasConn == null)
  {
   return Constants.NOT_CONNECTED;
  }
  //int iReturn = sendRawXML(STREAM_END);
  //anabasConn.disconnect();
  //anabasConn.stop();
  //return iReturn;
  return 1;
 }


 public int sendMsg(String type, String myNick, String ID, String body)
 {
  String sXML = type + "\0" + ID + "\0" + myNick + "\0" + body;
  //String sXML = type + "\0" + myNick + "\0" + ID + "\0" + body;

  return sendRawXML(sXML);
 }


 public int anabasChatJoin(String ID, String chatRoom, String chatServer)
 {
  String sXML = chatServer;
  System.out.println("sXML: " + sXML);
  return sendRawXML(sXML);
 }


 public int sendRawXML(String XML)
 {
  if (anabasConn == null)
  {
   return Constants.NOT_CONNECTED;
  }
  if (mbEchoXML)
    System.out.println("SENT: " + XML);

  boolean bRet = anabasConn.broadcastMessage(XML);

  if(mbEchoXML)
  {
   moClient.rawXML("\n==>\n" + XML + "\n<==\n");
  }
  if (bRet == true)
  {
   return Constants.SEND_SUCCEEDED;
  }
  else
  {
   return Constants.SEND_FAILED;
  }
 }

//===========================================================================
//                       P R I V A T E    M E T H O D S
//===========================================================================
 private String getID(String ID)
 {
  if (ID.length() > 0)
  {
   return ID;
  }
  else
  {
   //No ID supplied, so use a random one
   return "" + (int) ((Math.random() * 10000) + 1000);
  }
 }
}
