/****************************************************************************
 * This class sends and receives data packets using GMS Streams to and from
 * Anabas Server.
 * @author Anabas, Kangseok Kim
 ****************************************************************************/
package com.anabas.pdasharedlet;

import com.anabas.concepts.*;
import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import com.anabas.gxo.*;
import com.anabas.naming.*;

import java.util.*;

public class AnabasConnection extends Thread
                              implements GMS_MessageListener
{
 private IAnabasConnectionClient moConnectionClient = null;
 private AnabasConnectionState   moConnState = null;

 private CommunicationService m_commService;
 private GMS_StreamPublisher  m_publisherForPrivate;
 private GMS_StreamSubscriber m_subscriberForPublic;
 private GMS_Stream           m_streamForPrivate;
 private GMS_Stream           stream;

 private String myName;
 private boolean m_isReady = false;
 private Object m_lock = new String();
 private Vector m_textChatListener = new Vector();
 private String duplicateName = "";
 private String duplicateText = ""; 

 // Each client creates a publisher to send message to server,
 // and also creates a subscriber to receive message from server.
 // Each user send his text message to server first,
 // and server will record it and broadcast to all clients.
 public AnabasConnection(IAnabasConnectionClient connectionClient)
 {
  //moConnState = new ConnectionState(ConnectionState.CLOSED);
  moConnectionClient = connectionClient;

  initialCommServices();
  synchronized (m_lock)
  {
   updateState();
  }
  //anyway,
  //in case update has something wrong, we should let user can receive message
  m_isReady = true;
 }


 public void initialCommServices()
 {
  // Grab the handle to the communication service and
  // get the handle to the event bus stream for this sharedlet.
  try {
     Context ctx = ContextManager.getInitialContext();
     m_commService = (CommunicationService)ctx.lookup("services/CommunicationService");


     m_streamForPrivate = m_commService.findStream("application/x-sharedlet-communication/private");
     if (m_streamForPrivate == null)
     { // Create the stream if it doesn't exist.
      m_streamForPrivate = m_commService.createStream(PdaSharedletInfo.g_sharedletMIME+"/private");
     }
     // create publisher
     // only server can listen from private stream
     m_publisherForPrivate = m_streamForPrivate.createPublisher();



     // Sharedlets should use their MIME type or some derivative of it
     // as the name of the stream to avoid namespace collisions.
     // Try to find an existing stream.
     stream = m_commService.findStream("application/x-sharedlet-communication/public");
     if (stream == null)
     {
      stream = m_commService.createStream(PdaSharedletInfo.g_sharedletMIME+"/public");
     }
     // create subscriber
     // server will broadcast message to all clients from public stream
     m_subscriberForPublic = stream.createSubscriber();
     // Set itself as the message listener waiting for incoming messages and
     // requests.
     m_subscriberForPublic.setMessageListener(this);
  }
  catch (Exception e) {
       LogManager.err("communicationSharedle", "Unable to get comm service inside textchat", e);
  }
 }


 // Do nothing.
 public void activate()
 {
  LogManager.log("View", getID() + " view activated");
 }


 // Do nothing.
 public void deactivate()
 {
  LogManager.log("View", getID() + " view deactivated");
 }


 // @return "application/x-sharedlet-docnav"
 public String getSharedletMIME()
 {
  return PdaSharedletInfo.g_sharedletMIME;
 }

 /*
 // Invoked when this sharedlet's role has changed.
 public void roleChanged(String newRole)
 {
  LogManager.log("TestSharedlet","Role changed to : "+newRole);
 }


 // Listen for remote capability changes.
 public void remoteCapabilityChanged(String capability,boolean enabled,UserID user)
 {
  LogManager.log("TestControlView", "Remote '" + capability + "' is now " +
                 enabled + " for user: " + user);
 }


 // Listen for capability changes and enable/disable appropriate set of buttons
 // changes.
 public void capabilityChanged(String capability,boolean enabled)
 {
  LogManager.log("TestControlView", "My '" + capability+"' is now " + enabled);
 }


 // Invoked by SessionManager service when a user joins.
 // Need to update the user list.
 public void userJoined(SessionEvent e)
 {
  User u = (User)e.getSource();
  //m_userList.addItem(u.getUserID());
 }


 // Invoked by SessionManager service when a user leaves.
 // Need to update the user list.
 public void userLeft(SessionEvent e)
 {
  User u = (User)e.getSource();
  //m_userList.removeItem(u.getUserID());
 }
 */


 // Asynchronous reception of non-request messages
 public void onMessage(GMS_Message message)
 {
  if (!m_isReady) return;
  synchronized(m_lock)
  {
   if (message instanceof GMS_TextMessage)
   {
    GMS_TextMessage msg = (GMS_TextMessage)message;
    try {
       String messageType = (String)msg.getProperty("messagetype");
       if (messageType.equals("textchat"))
       {
        String userName = (String)msg.getProperty("username");
        String userID = (String)msg.getProperty("userid");
        if (duplicateName.equals(userName) && duplicateText.equals(msg.getText())) return;
        if (!userName.equals(myName))  //m_myUser.getUserID().getID()))
        {
         String sResponse = msg.getText();
         duplicateName = userName;
         duplicateText = sResponse;

         if (sResponse != null)
         {
          dataReceived(userName + ": ", sResponse);
          fireNewTextChatEvent();
         }
        }
       }
    }
    catch (GXO_Exception e) {
         LogManager.err("Textchat", "Unable to extract the message", e);
    }
   }
  }
 }


 private void fireNewTextChatEvent()
 {
  for (int i= 0; i < m_textChatListener.size(); i++)
  {
   TextChatListener tcl = (TextChatListener)m_textChatListener.elementAt(i);
   tcl.newTextChatComing();
  }
 }


 public void addTextChatListener(TextChatListener tcl)
 {
  m_textChatListener.addElement(tcl);
 }


 // The message type used in broadcast is GMS_TextMessage with 3 properties:
 // "messagetype", "username", "userid"  and textchat message
 public synchronized boolean broadcastMessage(String str)
 {
  StringTokenizer st = new StringTokenizer(str, "\0");

  String messageType = st.nextToken();
  String userName = st.nextToken();
  String userID = st.nextToken();
  String messageText = st.nextToken();

  myName = userName;

  try {
     GMS_TextMessage msg = (GMS_TextMessage)m_publisherForPrivate.createMessage(GMS_Message.MSGTYPE_ADMINISTRATION, GMS_Message.BODYTYPE_TEXT);


     msg.setProperty("messagetype", messageType);
     msg.setProperty("username", userName);
     msg.setProperty("userid", userID); //m_myUser.getUserID().getID());
     msg.setText(messageText);
     m_publisherForPrivate.broadcast(msg);

     return true;
  }
  catch (GXO_Exception ex) {
       LogManager.err("Textchat", "Unable to broadcast", ex);
       return false;
  }
 }


 // These function are not used. Asynchronous delivery of requests
 public void onRequest(GMS_Message m, GMS_MessageReplyer r){}


 // New user will send a request to get the text chat that already existed
 // in this meeting.
 // Here is the code example to send update message to server
 // and deal with the message replied by server.
 private void updateState()
 {
  GMS_TextMessage response = null;
  try {
     GMS_TextMessage requestMsg = (GMS_TextMessage)m_publisherForPrivate.createMessage(GMS_Message.MSGTYPE_ADMINISTRATION, GMS_Message.BODYTYPE_TEXT);
     requestMsg.setText("request");
     GMS_MulticastDestinationSelector selector = (GMS_MulticastDestinationSelector)m_publisherForPrivate.createDestinationSelector("GMS_MulticastDestinationSelector");
     GMS_Destination dst = m_streamForPrivate.findDestination(selector);
     response = (GMS_TextMessage)m_publisherForPrivate.request(requestMsg, dst, 1000);
     m_isReady = true;
     if (response != null)
     {
      String text = response.getText();
      /*
      StringTokenizer st = new StringTokenizer(text, "\0");
      while (st.hasMoreTokens())
      {
       String t = st.nextToken();
      */
      if (text.length() > 0)
        dataReceived("", text);
      //}
     }// otherwise, we suppose no text chat published before
  }
  catch (GXO_Exception e) {
       LogManager.err("CommunicationSharedlet", "Unable to update sessionLogic state", e);
  }
 }


 // @return The unique name of the control view.
 public String getID()
 {
  return "Test Control";
 }


 private void dataReceived(String name, String message)
 {
  moConnectionClient.dataReceived(name, message);
 }


 private void error(String str)
 {
   moConnectionClient.error(str);
 }
}

/****************************************************************************
 *                                 The End
 ****************************************************************************/

