/**
 * Title:        AdaptorSharedletInfo.java
 * Description:  Adaptor Sharedlet Info Class
 * Copyright:    Copyright (c) 2001
 * Company:      CSIT FSU
 * @author       Sangyoon Oh soh@csit.fsu.edu
 * @version 1.0
 */

package com.anabas.adaptorsharedlet;

import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import java.awt.Dimension;
import java.util.Vector;

public class AdaptorSharedletInfo extends SharedletRCUtil implements SharedletInfo {

  /* ======================= Member Variables =========================== */

  /**
   * The unique MIME type for the sharedlet
   */
  public static String g_sharedletMIME = "application/x-sharedlet-adaptor";

  /**
   * This sharedlet supports all document types.  When the loadDocument() method
   * is called, it will just display the action to stdout.
   */
  private static final String[] s_supportedDocTypes = { "*" };

  /**
   * Stores an internal vector of all SharedletViewInfo's
   */
  private Vector m_viewInfos = new Vector();

  /**
   * Stores an internal vector of all SharedletSessionLogicInfo's
   */
  private Vector m_logicInfos = new Vector();


  /* ==================== Methods =================================== */

  /**
   * Constructs 3 view infos, one for each view type
   * (Application,Contro,Presentation).
   *
   * Also setup the list of capabilities, roles, and moderatable elements.
   */
  public AdaptorSharedletInfo() {

    // Setup the list of session logic infos.
    m_logicInfos.addElement(new AdaptorSessionLogicInfo());
  }

  /**
   * @return "Adaptor Sharedlet"
   */
  public String   getDescription() {
    return "Session Document Navigator";
  }

  /**
   * @return application/x-sharedlet-docnav
   */
  public String   getMIMEType()   {
    return g_sharedletMIME;
  }

  /**
   * @return 1.0
   */
  public double   getVersion() {
    return 1.0;
  }

  /**
   * @return The list of view info that was setup in the constructor.
   */
  public Vector getViewInfos() {
    return m_viewInfos;
  }

  /**
   * @return The list of logic info that was setup in teh constructor.
   */
  public Vector getLogicInfos() {
    return m_logicInfos;
  }

  /**
   * @return The list of supported document types which is "*".
   */
  public String[] getSupportedDocumentTypes() {
    return s_supportedDocTypes;
  }
}