/****************************************************************************
 * This class sends and receives data packets using Input/Output Streams
 * @author	Jonathan Paulson
 ****************************************************************************/
package com.anabas.pdasharedlet;

import java.net.*;
import java.io.*;
import java.util.*;

public class Connection extends Thread
{
 private IConnectionClient moConnectionClient = null;
 private Socket		   moSocket = null;
 private InputStream	   moInputStream = null;
 private OutputStream	   moOutputStream = null;
 private ConnectionState   moConnState = null;

 public Connection(IConnectionClient connectionClient)
 {
  moConnState = new ConnectionState(ConnectionState.CLOSED);
  moConnectionClient = connectionClient;
 }

 public void run()
 {
  while (true)
  {
   if (moConnState.state() == moConnState.OPEN)
   {
    try {
       //Look for Data from Server
       byte b[] = new byte[1];
       moInputStream.read(b);
       if (b != null)
       {
	String sResponse = new String(b, 0, 1);
	dataReceived(sResponse);
       }
       else
       {
	disconnect();
       }
    }
    catch (EOFException e) {
         disconnect();
    }
    catch (IOException e) {
         disconnect();
    }
    //Don't hog processor
    try {
       sleep(1);
    }
    catch (InterruptedException e){}
   }
   else
   {
    //Don't hog processor
    try {
       sleep(1000);
    }
    catch (InterruptedException e){}
   }
  }
 }

 public synchronized boolean Send(String str)
 {
  if (moConnState.state() != moConnState.OPEN)
  {
   return false;
  }
  if (moOutputStream != null)
  {
   try {
      moOutputStream.write(str.getBytes());
      return true;
   }
   catch (IOException e) {
	error("Unable send Data");
	return false;
   }
  }
  else
  {
   error("Unable send Data, connection closed.");
   return false;
  }
 }

 public ConnectionState connect(String host, int port)
 {
  if (moSocket != null)
  {
   disconnect();
  }

  moConnState.state(ConnectionState.CONNECTION_PENDING);

  //Hook up socket Connection
  try {
     moSocket = new Socket(host, port);
     moConnState.state(moConnState.CONNECTING);
  }
  catch (UnknownHostException e) {
       error("UnknownHost " + host + ":" + port);
       moSocket = null;
       moConnState.state(moConnState.ERROR);
       return moConnState;
  }
  catch (IOException e) {
       error("Unable to connect to " + host + ":" + port);
       moSocket = null;
       moConnState.state(moConnState.ERROR);
       return moConnState;
  }

  try {
     //Hookup our reading and writting pipes to the host
     moInputStream = moSocket.getInputStream();
     moOutputStream = moSocket.getOutputStream();
     moConnState.state(moConnState.OPEN);
  }
  catch (IOException e) {
       moOutputStream = null;
       moInputStream = null;
       moConnState.state(moConnState.ERROR);
       return moConnState;
  }
  return moConnState;
 }

 public void disconnect()
 {
  moConnState.state(moConnState.CLOSING);
  try {
     if (moOutputStream != null)
     {
      moOutputStream.close();
     }
     if (moInputStream != null)
     {
      moInputStream.close();
     }
     if (moSocket != null)
     {
      moSocket.close();
     }
  }
  catch (IOException e) {}

  if (moConnectionClient != null)
  {
    moConnectionClient.connectionClosed();
  }
  moSocket = null;
  moOutputStream = null;
  moInputStream = null;
  moConnState.state(moConnState.CLOSED);
 }

 private void dataReceived(String str)
 {
  if (moConnectionClient != null)
  {
   moConnectionClient.dataReceived(str);
  }
 }

 private void error(String str)
 {
  if (moConnectionClient != null)
  {
   moConnectionClient.error(str);
  }
 }
}
