package com.anabas.svgsharedlet;

import com.anabas.concepts.*;
import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import com.anabas.gxo.*;
import com.anabas.naming.*;

import java.net.URL;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Vector;

import java.io.File;
import java.io.FileInputStream;
import com.anabas.util.nativeutils.NativeUtilsWrapper;
import java.util.Properties;
import java.util.Hashtable;
import java.io.IOException;

//import org.apache.batik.css.CSSDocumentHandler;
//import org.apache.batik.util.gui.resource.ResourceManager;

/**
 * This contains the core logic for SVG sharedlet.
 *
 * @author Sung-Hoon Ko, John Yin
 * @version 1.0
 */

public class SVGControlView extends JPanel implements  SharedletView,
                                                       ActionListener,
                                                       MouseListener,
                                                       GMS_MessageListener,
                                                       SessionListener,
                                                       CapabilityListener {

  private SharedletViewRenderer m_renderer;
  private CapabilitiesManager  m_capabilitiesManager;
  private SessionManager       m_sessionManager;
  private CommunicationService m_commService;
  private LauncherService      m_launcherService;
  private GMS_Stream           stream;
  private GMS_StreamPublisher  m_publisher;
  private GMS_StreamSubscriber m_subscriber;
  private static boolean canIopen = false;
  private static boolean isBrowserOpened = false;
  private User m_myUser;
  private String m_myUserName;

  // Used to open batik SVG browser; these for PC to PC not to PDA
  //private org.apache.batik.apps.svgbrowser.Main batikMain;
  //private org.apache.batik.apps.svgbrowser.JSVGViewerFrame batikFrame;

  public org.apache.batik.apps.svgbrowser.Main batikMain ;

  /* ================= UI Objects ================================ */

  private JTextField m_urlTextField = new JTextField(20);

  // Button that launch Batik browser.
  //public static JButton m_actionButton;
  private JButton m_actionButton;
  // Shows the user list
  private JComboBox m_userList = new JComboBox();
  // Give the ability to generate random number to a user
  private JButton m_giveButton = new JButton();
  // Take away the ability to generate random number ffrom a user.
  private JButton m_takeButton = new JButton();
  // Promote another user to Moderator.  Allowing that user to
  // remotely take/give random number generation control.
  private JButton m_promoteButton = new JButton();
  // Disallow another user to remotely take/give random number generation control.
  private JButton m_demoteButton = new JButton();
  // Open a file.  Used for testing the loadDocument/document support loop.
  private JButton m_openButton = new JButton();
  // File chooser to choose a file.
  private JFileChooser m_fileChooser = new JFileChooser();


  //////////////////////////////////////////////
  public SVGControlView() {
    // Grab the handle to the communication service and get the handle to the
    // event bus stream for this sharedlet.
    try {

      Context ctx = ContextManager.getInitialContext();
      m_commService = (CommunicationService)ctx.lookup("services/CommunicationService");

      // Sharedlets should use their MIME type or some derivative of it
      // as the name of the stream to avoid namespace collisions.
      // Try to find an existing stream.
      stream = m_commService.findStream(SVGSharedletInfo.g_sharedletMIME);
      //stream = m_commService.findStream("application/x-sharedlet-communication/public");

      // If the stream doesn't exist then create it.
      if (stream == null) {
        stream = m_commService.createStream(SVGSharedletInfo.g_sharedletMIME);
      }

      // Get a handle to the launcher server for launch files.
      m_launcherService = (LauncherService)ctx.lookup("services/LauncherService");


      // Create a publisher & subscriber to the stream so 
      // it can receive and send messages to the stream.
      m_publisher = stream.createPublisher();
      m_subscriber = stream.createSubscriber();

      // Set itself as the message listener waiting for incoming messages and
      // requests.
      m_subscriber.setMessageListener(this);

      // Get a handle to the capabilities manager so it can check for 
      // what permssions this user has.
      m_capabilitiesManager = (CapabilitiesManager)ctx.lookup("services/CapabilitiesManager");
      // Needs to be a listener to know when capabilities change so it can enable/disable
      // certain buttons.
      m_capabilitiesManager.addCapabilityListener(this);

      Vector moderatables = m_capabilitiesManager.getModeratableElements(SVGSharedletInfo.g_sharedletMIME);
      Enumeration enum = moderatables.elements();
      while(enum.hasMoreElements()) {
        String me = (String)enum.nextElement();
        LogManager.log("SVGSharedlet","Visible Moderatable Element: " + me);
      }

      // Get a handle to the session manager and act as a listener to 
      // update the user list selector.
      m_sessionManager = (SessionManager)ctx.lookup("services/SessionManager");
      m_sessionManager.addSessionListener(this);

      // Get user info.
      m_myUser = m_sessionManager.getMyUser();
      m_myUserName = m_myUser.getFirstName();

      // Initialize the user list selector.
      Vector p = m_sessionManager.getParticipants();
      enum = p.elements();
      while(enum.hasMoreElements()) {
        m_userList.addItem(((User)enum.nextElement()).getUserID());
      }
    } catch (Exception e) {
      LogManager.err("SVGControlView","Unable to get comm service",e);
    }

    m_renderer = new JavaViewRenderer(this,this);

    try {
      jbInit();
    } catch (Exception e) {
    }
  }

  /**
   * Do nothing.
   */
  public void activate() {
    canIopen = canOpen();
    LogManager.log("View",getID() + " view activated");
  }

  /**
   * Do nothing.
   */
  public void deactivate() {
    LogManager.log("View",getID() + " view deactivated");
  }

  /**
   * Should be triggered by a user clicking on the "Open" button and choosing
   * a file.
   */
  public void loadDocument(java.net.URL documentURL) {
    LogManager.log("View",getID() + " load document: " + documentURL);
  }



  /**
   * @return "application/x-sharedlet-docnav"
   */
  public String getSharedletMIME() {
    return SVGSharedletInfo.g_sharedletMIME;
  }

  /**
   * Invoked when this sharedlet's capability manager has changed modes.
   * Not currently used; however, it could alternatively be used to manage
   * the moderator control rather than manually doing it.
   */
  /*public void modeChanged(short newMode) {
    LogManager.log("SVGSharedlet","Capabilities manager mode changed to: " + newMode);
  }*/

  /**
   * Invoked when this sharedlet's role has changed.
   */
  public void roleChanged(String newRole) {
    LogManager.log("SVGSharedlet","Role changed to : "+newRole);
  }

  /**
   * Listen for remote capability changes.
   */
  public void remoteCapabilityChanged(String capability,boolean enabled,UserID user) {
    LogManager.log("SVGControlView","Remote '"+capability+"' is now " +
                   enabled + " for user: " + user);
  }

  /**
   * Listen for capability changes and enable/disable appropriate set of buttons.
   * changes.
   */
  public void capabilityChanged(String capability,boolean enabled) {
    LogManager.log("SVGControlView","My '"+capability+"' is now " + enabled);
    if (capability.equals("1:browser")) {
      m_actionButton.setEnabled(enabled);
    } else if (capability.equals("give")) {
      m_giveButton.setEnabled(enabled);
    } else if (capability.equals("take")) {
      m_takeButton.setEnabled(enabled);
    } else if (capability.equals("moderate")) {
      m_promoteButton.setEnabled(enabled);
      m_demoteButton.setEnabled(enabled);
    }
  }

  /**
   * Invoked by SessionManager service when a user joins.  Need to
   * update the user list.
   */
  public void userJoined(SessionEvent e) {
    User u = (User)e.getSource();
    m_userList.addItem(u.getUserID());
  }

  /**
   * Invoked by SessionManager service when a user leaves.  Need to
   * update the user list.
   */
  public void userLeft(SessionEvent e) {
    User u = (User)e.getSource();
    m_userList.removeItem(u.getUserID());
  }

  /** 
   * Broadcast the location of SVG file.
   */

  public void svgLocationPass(String location) {
    try {
      // Create a new text message to contain the location of SVG file.
      GMS_TextMessage msg = (GMS_TextMessage) m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,GMS_Message.BODYTYPE_TEXT);
      
       msg.setProperty("username", m_myUserName);
       msg.setProperty("userid",m_myUser.getUserID().getID());

      // Set the location into the message.
      msg.setText("1"+location);  // add 1 to mark event tag
      
      // Broadcast onto the stream.
      m_publisher.broadcast(msg);
      
      LogManager.log("SVGControlView","Broadcasted " + msg.getText()); 
    } catch (GXO_Exception ex) {
        LogManager.err("TextSharedlet","Unable to broadcast",ex);
    }

  }



  /** 
   * Broadcast the action of SVG.
   */

  public void svgActionPass(String action) {
    try {
      // Create a new text message to contain the location of SVG file.
      GMS_TextMessage msg = (GMS_TextMessage) m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,GMS_Message.BODYTYPE_TEXT);
      
      msg.setProperty("username", m_myUserName);
      msg.setProperty("userid",m_myUser.getUserID().getID());
      msg.setText(action); 
      
      // Broadcast onto the stream.
      m_publisher.broadcast(msg);
      
      LogManager.log("SVGControlView","Broadcasted " + msg.getText()); 
    } catch (GXO_Exception ex) {
        LogManager.err("TextSharedlet","Unable to broadcast",ex);
    }

  }


  /**
   * Receives a message
   */
  public void onMessage(GMS_Message m) {
    try {
      GMS_TextMessage msg = (GMS_TextMessage)m;
      
      String str = msg.getText();
      LogManager.log("SVGControlView","got onMessage!!!!!!!!! " + str); 

      String userName = (String)msg.getProperty("username");
      String userID = (String)msg.getProperty("userid");


      //String realAction = str.substring(1);
      //LogManager.log("SVGControlView","got onMessage 222 !!!!!!!!! " + realAction); 
      if(!userID.equals(m_myUser.getUserID().getID())){
        if(str.equals("LAUNCH")) 
          if (!isBrowserOpened) {
            isBrowserOpened = true;
            m_actionButton.setEnabled(false);
            String realLocation = str.substring(1);
            String[] args3 ={realLocation};
            LogManager.log("Batik launch","MyName: "+m_myUserName);

            String[] args ={""};
            batikMain = new org.apache.batik.apps.svgbrowser.Main(args);
            batikMain.run();
          }
          else LogManager.log("Batik Not launch","MyName: "+m_myUserName);
        else {
          String subStr = str.substring(0,1);
          //String[] args3 ={realLocation};       
          if(subStr.equals("1")) {
            LogManager.log("LOCATION SENT","MyName: "+m_myUserName);            
            batikMain.frame.getJSVGCanvas().loadSVGDocument(str.substring(1));
          }
        }
      }
      //else
        if(str.equals("ENABLEBUTTON")) {
          batikMain.frame.setVisible(false);
          isBrowserOpened = false;
          m_actionButton.setEnabled(true);
          
          /*
          if (!m_capabilitiesManager.isCapable(SVGSharedletInfo.g_sharedletMIME,"1:browser")) 
            m_actionButton.setEnabled(false);            
            */
          
        }
        /*
        if (!canIopen && !isBrowserOpened) {
          isBrowserOpened = true;
          String realLocation = str.substring(1);
          String[] args3 ={realLocation};
          org.apache.batik.apps.svgbrowser.Main batikMain3 = new org.apache.batik.apps.svgbrowser.Main(args2);
          LogManager.log("Batik launch","*************");
          batikMain3.run();
        }
        */

      // Set new location address in batic browser. 
      // do only for PDA this time
      //batikMain.arguments[0] = str;
      //batikMain.aaa = str;
      //batikMain.run();

      //org.apache.batik.apps.svgbrowser.JSVGViewerFrame batikFrame = new org.apache.batik.apps.svgbrowser.Main(args2);
      //batikFrame = new org.apache.batik.apps.svgbrowser.Main(args2);
      /*
        JSVGViewerFrame f = createAndShowJSVGViewerFrame();
        f.getJSVGCanvas().loadSVGDocument(url);
        */

    } catch (GXO_Exception e) {
      LogManager.err("SVGControlView","error on message",e);
    }
  }

  /**
   * Main trap for various button actions.
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand().equals("Open Browser")) {
      // Launch Batik SVG Browser
      //String[] args2 = {"C:\\Batik\\samples\\anne.svg"};
      //String[] args2 = {"http://aspen.csit.fsu.edu/project/cctools/SVG/image1.svgz"};
      //String[] args2 ={""};
      //org.apache.batik.apps.svgbrowser.Main batikMain = new org.apache.batik.apps.svgbrowser.Main(args2);
      svgActionPass("LAUNCH");  // wakes up invitee's browsers first.
      m_actionButton.setEnabled(false);
      LogManager.log("Batik launch","%%%%%%%%%%%%%%%");
      String[] args ={""};
      batikMain = new org.apache.batik.apps.svgbrowser.Main(args);
      batikMain.run();
      isBrowserOpened = true;

    } else if (e.getActionCommand().equals("givecontrol")) {
      try {
        LogManager.log("SVGControlView","Enabling capability on moderation to" +
                       ((UserID)m_userList.getSelectedItem()).toString());
        // Use the capabilities manager API to enable a remote user's
        // "Application Control" moderatable element for this sharedlet.
        // The remote capability manager will automatically find the mapping
        // to the list of capabilities and change them thus firing off
        // capability events.
        m_capabilitiesManager.enableRemoteModeratable(SVGSharedletInfo.g_sharedletMIME,
                                                      "Application Control",
                                                      (UserID)m_userList.getSelectedItem());
      } catch (PermissionDeniedException ex) {
        LogManager.err("SVGControlView","Can not give control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("takecontrol")) {
      try {
        LogManager.log("SVGControlView","Disabling capability on moderation to" +
                       ((UserID)m_userList.getSelectedItem()).toString());
        // Use the capabilities manager API to disable a remote user's
        // "Application Control" moderatable element for this sharedlet.
        // The remote capability manager will automatically find the mapping
        // to the list of capabilities and change them thus firing off
        // capability events.
        m_capabilitiesManager.disableRemoteModeratable(SVGSharedletInfo.g_sharedletMIME,
                                             "Application Control",
                                             (UserID)m_userList.getSelectedItem());
      } catch (PermissionDeniedException ex) {
        LogManager.err("SVGControlView","Can not take away control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("promote")) {
      try {
        UserID user = (UserID)m_userList.getSelectedItem();
        String remoteRole = m_capabilitiesManager.getRemoteApplicationRole(user);
        String newRole = "";
        if (remoteRole.equals("host")) {
         LogManager.log("SVGControlView","User " + ((UserID)m_userList.getSelectedItem()).toString()+
                        " already host...no further promotion possible");
        } else if (remoteRole.equals("moderator")) {
         LogManager.log("SVGControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " being promoted to host");
         newRole = "host";
        } else if (remoteRole.equals("participant")) {
         LogManager.log("SVGControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " being promoted to moderator");
         newRole = "moderator";
        }

        // Promote another user to moderator mode.
        // First we have to set the remote capability manager to Moderator Mode
        // so that it can remotely set other people's capabilities.
        if (newRole.length() > 0) {
          m_capabilitiesManager.setRemoteApplicationRole(user,newRole);
        }
      } catch (PermissionDeniedException ex) {
        LogManager.err("SVGControlView","Can not give control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("demote")) {
      try {
        UserID user = (UserID)m_userList.getSelectedItem();
        String remoteRole = m_capabilitiesManager.getRemoteApplicationRole(user);
        String newRole = "";
        if (remoteRole.equals("host")) {
         LogManager.log("SVGControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " demoting to moderator");
         newRole = "moderator";
        } else if (remoteRole.equals("moderator")) {
         LogManager.log("SVGControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " being demoted to participant");
         newRole = "participant";
        } else if (remoteRole.equals("participant")) {
         LogManager.log("SVGControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " is already the lowest of the low.");
        }

        // Promote another user to moderator mode.
        // First we have to set the remote capability manager to Moderator Mode
        // so that it can remotely set other people's capabilities.
        if (newRole.length() > 0) {
          m_capabilitiesManager.setRemoteApplicationRole(user,newRole);
        }
      } catch (PermissionDeniedException ex) {
        LogManager.err("SVGControlView","Can not give control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("open")) {
      // Opens a file.
      File openedFile = m_fileChooser.getSelectedFile();
      int returnVal = m_fileChooser.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        openFile(openedFile);
      }
    }
  }

  /**
   * Invokes the LauncherService's launchDocument.  Since this sharedlet supports
   * "*" document types, the SVGSharedlet.loadDocument() should get invoked.
   */
  // private void openFile(String filename, String pathname) {
  private void openFile(File file) {
    try {
      //--m_launcherService.launch(new URL("file","localhost",filename));
      //m_launcherService.launch(new URL("file",pathname,filename));
      m_launcherService.launch(new URL("file","",file.getPath()));
    } catch (Exception e) {
      LogManager.err("Control View","Malformed URL: " + file.getName());
    }
  }


  private boolean canOpen() {
    return m_capabilitiesManager.isCapable(SVGSharedletInfo.g_sharedletMIME, "1:browser");
  }

  /**
   * These function are not used.
   */
  public void onRequest(GMS_Message m,GMS_MessageReplyer r) {}
  public void mouseClicked(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {};
  public void mouseReleased(MouseEvent e) {};
  public void mouseEntered(MouseEvent e) {};
  public void mouseExited(MouseEvent e) {};
  public void mouseDragged(MouseEvent e) {};

  /**
   * @return The unique name of the control view.
   */
  public String getID() {
    return "SVG Control";
  }

  /**
   * @return The Java renderer
   */
  public SharedletViewRenderer getRenderer(String MIMEType) {
    if (MIMEType.equals("application/java")) {
      return m_renderer;
    }
    return null;
  }

  /**
   * Initializes the control view layout.  Just a simple flow layout.
   */
  private void jbInit() throws Exception {
    setLayout(new FlowLayout());

    Dimension d = new Dimension(150,150);
    this.setPreferredSize(d);

    //m_actionButton = new JButton("Start");
    m_actionButton = new JButton("Open Browser");


    m_giveButton.setActionCommand("givecontrol");
    m_giveButton.setText("Give");
    m_takeButton.setActionCommand("takecontrol");
    m_takeButton.setText("Take");
    m_promoteButton.setActionCommand("promote");
    m_promoteButton.setText("Promote");
    m_demoteButton.setActionCommand("demote");
    m_demoteButton.setText("Demote");
    m_openButton.setActionCommand("open");
    m_openButton.setText("Open");
    /*
    add(m_actionButton);

    this.add(m_userList, null);
    this.add(m_giveButton, null);
    this.add(m_takeButton, null);
    this.add(m_promoteButton, null);
    this.add(m_demoteButton, null);
    this.add(m_openButton, null);
    this.addMouseListener(this);
    */
    //this.add(m_urlTextField, null);
    add(m_actionButton);
    //this.add(m_giveButton, null);
    //this.add(m_openButton, null);
    
    m_actionButton.addActionListener(this);
    m_giveButton.addActionListener(this);
    m_takeButton.addActionListener(this);
    m_promoteButton.addActionListener(this);
    m_demoteButton.addActionListener(this);
    m_openButton.addActionListener(this);
    m_fileChooser.addActionListener(this);
    
    /*
    if (!m_capabilitiesManager.isCapable(SVGSharedletInfo.g_sharedletMIME,"1:browser")) {
      m_actionButton.setEnabled(false);
     }
     */

    if (!m_capabilitiesManager.isCapable(SVGSharedletInfo.g_sharedletMIME,"take")) {
      m_takeButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(SVGSharedletInfo.g_sharedletMIME,"give")) {
      m_giveButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(SVGSharedletInfo.g_sharedletMIME,"moderate")) {
      m_promoteButton.setEnabled(false);
      m_demoteButton.setEnabled(false);
    }
  }


}
