package com.anabas.svgsharedlet;

import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import java.awt.Dimension;
import java.util.Vector;

/**
 * The sharedlet info structure for SVGSharedlet.
 *
 * @author Sung-Hoon Ko, John Yin
 * @version 1.0
 */

public class SVGSharedletInfo extends SharedletRCUtil implements SharedletInfo {

  /* ======================= Member Variables =========================== */

  /**
   * The unique MIME type for the sharedlet
   */
  public static String g_sharedletMIME = "application/x-sharedlet-garnetsvg";

  /**
   * This sharedlet supports all document types.  When the loadDocument() method
   * is called, it will just display the action to stdout.
   */
  private static final String[] s_supportedDocTypes = { "*" };

  /**
   * Stores an internal vector of all SharedletViewInfo's
   */
  private Vector m_viewInfos = new Vector();

  /**
   * Stores an internal vector of all SharedletSessionLogicInfo's
   */
  private Vector m_logicInfos = new Vector();


  /* ==================== Methods =================================== */

  /**
   * Constructs 3 view infos, one for each view type
   * (Application,Control,Presentation).
   *
   * Also setup the list of capabilities, roles, and moderatable elements.
   */
  public SVGSharedletInfo() {

    // Setup the list of view infos.
    //--m_viewInfos.addElement(new ApplicationViewInfo());
    m_viewInfos.addElement(new SVGControlViewInfo());
    //m_viewInfos.addElement(new PresentationViewInfo());
    //m_viewInfos.addElement(new TokenTestViewInfo());

    // Setup the list of session logic infos.
    m_logicInfos.addElement(new SVGSessionLogicInfo());

    /**
     * Setup the list of raw capabilities.
     */
    setCapabilities("1:browser,take,give,moderate");

    /**
     * The "Application Control" element means toggling the start capability
     */
    setModeratable("Application Control","1:browser");


    /**
     * A participant starts with no capabilities and can not see the application view.
     */
    setRole("Participant","","SVG Browser","");

    /**
     * A host starts with all possible capabilities and can see all views.
     */
    setRole("Host","1:browser,take,give,moderate","SVG Browser","Application Control");

    /**
     * A moderator has everything but the ability to pass on moderation control.
     */
    setRole("Moderator","1:browser,take,give","SVG Browser","Application Control");

  }

  /**
   * @return "Session Document Navigator"
   */
  public String   getDescription() {
    return "Session Document Navigator";
  }

  /**
   * @return application/x-sharedlet-docnav
   */
  public String   getMIMEType()   {
    return g_sharedletMIME;
  }

  /**
   * @return 1.0
   */
  public double   getVersion() {
    return 1.0;
  }

  /**
   * @return The list of view info that was setup in the constructor.
   */
  public Vector getViewInfos() {
    return m_viewInfos;
  }

  /**
   * @return The list of logic info that was setup in teh constructor.
   */
  public Vector getLogicInfos() {
    return m_logicInfos;
  }

  /**
   * @return The list of supported document types which is "*".
   */
  public String[] getSupportedDocumentTypes() {
    return s_supportedDocTypes;
  }
}
