/****************************************************************************
 * This class handles only basic Anabas server communication.
 * @author  Kangseok Kim
 ****************************************************************************/
package com.anabas.pdasharedlet;

import java.util.*;

public class AnabasComm implements IAnabasConnectionClient
{
 private IAnabasClient       moClient       = null;
 private AnabasConnection    anabasConn     = null;
 private static final String DEFAULT_HOST   = "sycamore.csit.fsu.edu";
 private static final int    DEFAULT_PORT   = 443;


 // @param client Reference to the Object that implements IAnabasClient
 //               Used to communicate back to the object
 public AnabasComm(IAnabasClient client)
 {
  moClient = client;
 }

//================= Required IAnabasConnectionClient methods =================

 // Raw Data received from host
 public void dataReceived(String name, String body)
 {
  moClient.anabasMsgReceived(name, body);
 }


 public void connectionClosed()
 {
  moClient.connectionClosed();
  moClient.rawXML("\nDisconnected");
 }


 public void error(String str)
 {
  moClient.rawXML(str);
  moClient.error(str);
 }
//============================================================================

 // Connect to Anabas server
 public void connect(String host, String hostName, String port)
 {
  anabasConn = new AnabasConnection(this);
  anabasConn.start();
 }


 // Disconnect from Anabas Server
 public int disconnect()
 {
  if (anabasConn == null)
  {
   return Constants.NOT_CONNECTED;
  }

  return 1;
 }


 public int sendMsg(String type, String myNick, String ID, String body)
 {
  String msg = type + "\0" + myNick + "\0" + ID + "\0" + body;

  return sendRawMessage(msg);
 }


 public int anabasChatJoin(String type, String myNick, String ID, String body)
 {
  String msg = type + "\0" + myNick + "\0" + ID + "\0" + body;

  return sendRawMessage(msg);
 }


 // Sends the supplied Message (without modifing it in any way) to the
 // Anabas Server
 public int sendRawMessage(String Msg)
 {
  if (anabasConn == null)
  {
   return Constants.NOT_CONNECTED;
  }

  boolean bRet = anabasConn.broadcastMessage(Msg);

  if (bRet == true)
  {
   return Constants.SEND_SUCCEEDED;
  }
  else
  {
   return Constants.SEND_FAILED;
  }
 }

//===========================================================================
//                       P R I V A T E    M E T H O D S
//===========================================================================
 private String getID(String ID)
 {
  if (ID.length() > 0)
  {
   return ID;
  }
  else
  {
   //No ID supplied, so use a random one
   return "" + (int) ((Math.random() * 10000) + 1000);
  }
 }
}
