package com.anabas.pdasharedlet;

import com.anabas.concepts.*;
import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import com.anabas.gxo.*;
import com.anabas.naming.*;

import com.anabas.sdsharedlet.*;

import java.net.URL;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Vector;

import java.io.File;
import java.io.FileInputStream;
import com.anabas.util.nativeutils.NativeUtilsWrapper;
import java.util.Properties;
import java.util.Hashtable;
import java.io.IOException;

/**
 * This contains the core logic for this pda sharedlet.
 *
 * @author John Yin
 *
 * @modified by Sangyoon Oh
 * @version 1.1
 */

public class PdaControlView extends JPanel implements SharedletView,
                                                       ActionListener,
                                                       MouseListener,
                                                       GMS_MessageListener,
                                                       SessionListener,
                                                       CapabilityListener {
  // Adaptor Module
  protected PAdaptor pa;

  // Listener Module
  protected GSDListener sdlistener;
  // private SVGListener svglistener;
  private TextChatListener textchatlistener;
  private SDSharedlet m_sdsharedlet;
  private SharedletManager m_sharedletManager;

  // Used to generate random number messages
  private static java.util.Random s_random = new java.util.Random(System.currentTimeMillis());

  private SharedletViewRenderer m_renderer;
  // protected PresentationView      m_presentationView;

  private CapabilitiesManager m_capabilitiesManager;
  private SessionManager      m_sessionManager;
  private CommunicationService m_commService;
  private LauncherService     m_launcherService;
  private GMS_StreamPublisher m_publisher;
  private GMS_StreamPublisher m_publishertextprivate;
  private GMS_StreamSubscriber m_subscriber;
  private GMS_StreamSubscriber m_subscriber_whiteboard;
  private GMS_StreamSubscriber m_subscriber_sd;
  private GMS_StreamSubscriber m_subscriber_svg;
  private GMS_Stream stream;
  private GMS_Stream stream_textprivate;
  private GMS_Stream stream_whiteboard;
  private GMS_Stream stream_sd;
  private GMS_Stream stream_svg;

  // SD
  private Hashtable m_ht_curURLs = new Hashtable();
  //private SDWrapper m_sdWrapper;
  private LayoutService m_layoutService;
  //private SDControlView m_controlView;

  /* ================= UI Objects ================================ */

  private JTextField m_urlTextField = new JTextField(20);

  // Button that triggers the generation of a random number.
  private JButton m_actionButton;
  // Shows the user list
  private JComboBox m_userList = new JComboBox();
  // Give the ability to generate random number to a user
  private JButton m_giveButton = new JButton();
  // Take away the ability to generate random number ffrom a user.
  private JButton m_takeButton = new JButton();
  // Promote another user to Moderator.  Allowing that user to
  // remotely take/give random number generation control.
  private JButton m_promoteButton = new JButton();
  // Disallow another user to remotely take/give random number generation control.
  private JButton m_demoteButton = new JButton();
  // Open a file.  Used for testing the loadDocument/document support loop.
  private JButton m_openButton = new JButton();
  // File chooser to choose a file.
  private JFileChooser m_fileChooser = new JFileChooser();

  public PdaControlView() {
    // Grab the handle to the communication service and get the handle to the
    // event bus stream for this sharedlet.
    try {
      Context ctx = ContextManager.getInitialContext();
      m_commService = (CommunicationService)ctx.lookup("services/CommunicationService");

      // Sharedlets should use their MIME type or some derivative of it
      // as the name of the stream to avoid namespace collisions.
      // Try to find an existing stream.
      stream = m_commService.findStream("application/x-sharedlet-communication/public");
      stream_textprivate=m_commService.findStream("application/x-sharedlet-communication/private");
      stream_whiteboard = m_commService.findStream("application/x-sharedlet-whiteboard/public");
      stream_sd = m_commService.findStream("application/x-sharedlet-sd");
      stream_svg = m_commService.findStream("application/x-sharedlet-garnetsvg");

      // If the stream doesn't exist then create it.
      if (stream == null) {
        stream = m_commService.createStream(PdaSharedletInfo.g_sharedletMIME);
      }

      // Get a handle to the launcher server for launch files.
      m_launcherService = (LauncherService)ctx.lookup("services/LauncherService");


      // Create a publisher & subscriber to the stream so it can receive and send
      // messages to the stream.
      m_publisher = stream.createPublisher();
      m_publishertextprivate = stream_textprivate.createPublisher();
      m_subscriber = stream.createSubscriber();
      m_subscriber_whiteboard = stream_whiteboard.createSubscriber();
      m_subscriber_sd = stream_sd.createSubscriber();
      m_subscriber_svg = stream_svg.createSubscriber();

      // Set itself as the message listener waiting for incoming messages and
      // requests.
      // m_subscriber.setMessageListener(this);
      // m_subscriber_svg.setMessageListener(this);
      m_subscriber_whiteboard.setMessageListener(this);
      m_subscriber_sd.setMessageListener(this);

      // Get a handle to the capabilities manager so it can check for what permssions
      // this user has.
      m_capabilitiesManager = (CapabilitiesManager)ctx.lookup("services/CapabilitiesManager");
      // Needs to be a listener to know when capabilities change so it can enable/disable
      // certain buttons.
      m_capabilitiesManager.addCapabilityListener(this);

      Vector moderatables = m_capabilitiesManager.getModeratableElements(PdaSharedletInfo.g_sharedletMIME);
      Enumeration enum = moderatables.elements();
      while(enum.hasMoreElements()) {
        String me = (String)enum.nextElement();
        LogManager.log("PdaSharedlet","Visible Moderatable Element: " + me);
      }

      // Get a handle to the session manager and act as a listener to update the
      // user list selector.
      m_sessionManager = (SessionManager)ctx.lookup("services/SessionManager");
      m_sessionManager.addSessionListener(this);

      // Initialize the user list selector.
      Vector p = m_sessionManager.getParticipants();
      enum = p.elements();
      while(enum.hasMoreElements()) {
        m_userList.addItem(((User)enum.nextElement()).getUserID());
      }
    } catch (Exception e) {
      LogManager.err("PdaControlView","Unable to get comm service",e);
    }

    m_renderer = new JavaViewRenderer(this,this);

    try {
      jbInit();
    } catch (Exception e) {
    }

    /**
     *  Initiate PDA Adaptor module
     */
    if(pa == null) pa = new PAdaptor(this);

    /**
    try {
      m_sharedletManager = (SharedletManager) ctx.lookup("services/SharedletManager");
      m_sdsharedlet = (SDSharedlet) m_sharedletManager.getSharedlet(
                          m_sharedletManager.getInfo(SDSharedletInfo.g_sharedletMIME));
    } catch (Exception e) {
      LogManager.err("PdaControlView","Unable to get SD service",e);
    }
    **/

    /**
     *  Initiate Listener module
     */
    if(svglistener == null) svglistener = new SVGListener(this, m_subscriber_svg);
    //if(sdlistener == null) sdlistener = new GSDListener(this, m_sdsharedlet);
    //if(textchatlistener == null) textchatlistener = new TextChatListener(this, m_subscriber);

 }

  /**
   * Do nothing.
   */
  public void activate() {
    LogManager.log("View",getID() + " view activated");
  }

  /**
   * Do nothing.
   */
  public void deactivate() {
    LogManager.log("View",getID() + " view deactivated");
  }

  /**
   * @return "application/x-sharedlet-docnav"
   */
  public String getSharedletMIME() {
    return PdaSharedletInfo.g_sharedletMIME;
  }

  /**
   * Invoked when this sharedlet's role has changed.
   */
  public void roleChanged(String newRole) {
    LogManager.log("PdaSharedlet","Role changed to : "+newRole);
  }

  /**
   * Listen for remote capability changes.
   */
  public void remoteCapabilityChanged(String capability,boolean enabled,UserID user) {
    LogManager.log("PdaControlView","Remote '"+capability+"' is now " +
                   enabled + " for user: " + user);
  }

  /**
   * Listen for capability changes and enable/disable appropriate set of buttons.
   * changes.
   */
  public void capabilityChanged(String capability,boolean enabled) {
    LogManager.log("PdaControlView","My '"+capability+"' is now " + enabled);
    if (capability.equals("1:start")) {
      m_actionButton.setEnabled(enabled);
    } else if (capability.equals("give")) {
      m_giveButton.setEnabled(enabled);
    } else if (capability.equals("take")) {
      m_takeButton.setEnabled(enabled);
    } else if (capability.equals("moderate")) {
      m_promoteButton.setEnabled(enabled);
      m_demoteButton.setEnabled(enabled);
    }
  }

  /**
   * Invoked by SessionManager service when a user joins.  Need to
   * update the user list.
   */
  public void userJoined(SessionEvent e) {
    User u = (User)e.getSource();
    m_userList.addItem(u.getUserID());
  }

  /**
   * Invoked by SessionManager service when a user leaves.  Need to
   * update the user list.
   */
  public void userLeft(SessionEvent e) {
    User u = (User)e.getSource();
    m_userList.removeItem(u.getUserID());
  }

  /**
   * When it receives a message from the bus containing a new random number
   * that was generated by another user, display it in the presentation area.
   */
  public void onMessage(GMS_Message m) {

    try {

        if(m instanceof GMS_TextMessage){
          GMS_TextMessage msg = (GMS_TextMessage)m;
          String messageType = (String)msg.getProperty("messagetype");
          if(messageType.equals("textchat")){

          String userName = (String)msg.getProperty("username");
          String userID = (String)msg.getProperty("userid");
          String text = (String)msg.getText();

          // put GMS_Message to PAdaptor
          pa.getGMSMessage(messageType, userName, userID, text, 4);
        }
        else if(messageType.equals("svg")){

            String userName = (String)msg.getProperty("username");
            String userID = (String)msg.getProperty("userid");
            String text = (String)msg.getText();

            // put GMS_Message to PAdaptor
            pa.getGMSMessage(messageType, userName, userID, text, 1);
        }
        else {
          System.out.println("from whiteboard");
        }
      }
      else {
        //m_presentationView.displayMessage("from SD");
        //if(m instanceof GMS_BytesMessage) {
        GMS_BytesMessage bm = (GMS_BytesMessage)m;
        System.out.println(bm.getBytes());
      }

    }
    catch (GXO_Exception e) {
      LogManager.err("PdaControlView","error on message",e);
    }
  }

  /**
  public void broadcastMessage(String messageType, String userName,
              String userID, byte[] messageBody, int applicationID){}
  **/

  public void broadcastMessage(String messageType, String userName,
              String userID, String messageText, int applicationID){
     /**
     switch(applicationID) {

      case '0':
        break;

      case '1':
        // svg.getGMSMessage(msg);
        System.out.println(messageType + " " + userName + " " + userID + " " + msg);
        break;
     }
     **/
     try {
       GMS_TextMessage msg = (GMS_TextMessage)m_publishertextprivate.createMessage(GMS_Message.MSGTYPE_MINOR,
                                                      GMS_Message.BODYTYPE_TEXT);
       msg.setProperty("messagetype", messageType);
       msg.setProperty("username", userName);
       msg.setProperty("userid",userID);
       msg.setText(messageText);
       m_publishertextprivate.broadcast(msg);
     } catch (GXO_Exception ex) {
         LogManager.err("Communication","Unable to broadcast",ex);
     }
   }

  /**
   * Main trap for various button actions.
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand().equals("Go")) {
      try {

        // Create a new text message to contain the new random number.
        GMS_TextMessage msg = (GMS_TextMessage)m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,
                                                      GMS_Message.BODYTYPE_TEXT);

        // Set the new random number into the message.
        msg.setText("Random Double Message: " + Double.toString(s_random.nextDouble()));

        // Broadcast onto the stream.
        m_publisher.broadcast(msg);

        LogManager.log("PdaControlView","Broadcasted " + msg.getText());

      } catch (GXO_Exception ex) {
        LogManager.err("TextSharedlet","Unable to broadcast",ex);
      }
    }
  }

  /**
   * These function are not used.
   */
  public void onRequest(GMS_Message m,GMS_MessageReplyer r) {}
  public void mouseClicked(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {};
  public void mouseReleased(MouseEvent e) {};
  public void mouseEntered(MouseEvent e) {};
  public void mouseExited(MouseEvent e) {};
  public void mouseDragged(MouseEvent e) {};

  /**
   * @return The unique name of the control view.
   */
  public String getID() {
    return "PDA";
  }

  /**
   * @return The Java renderer
   */
  public SharedletViewRenderer getRenderer(String MIMEType) {
    if (MIMEType.equals("application/java")) {
      return m_renderer;
    }
    return null;
  }

  /**
   * Initializes the control view layout.  Just a simple flow layout.
   */
  private void jbInit() throws Exception {
    setLayout(new FlowLayout());

    Dimension d = new Dimension(150,30);
    this.setPreferredSize(d);

    // m_actionButton = new JButton("Go");
    // m_actionButton.setVisible(true);
    // this.add(m_actionButton);
    // m_actionButton.addActionListener(this);

    if (!m_capabilitiesManager.isCapable(PdaSharedletInfo.g_sharedletMIME,"1:start")) {
      m_actionButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(PdaSharedletInfo.g_sharedletMIME,"take")) {
      m_takeButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(PdaSharedletInfo.g_sharedletMIME,"give")) {
      m_giveButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(PdaSharedletInfo.g_sharedletMIME,"moderate")) {
      m_promoteButton.setEnabled(false);
      m_demoteButton.setEnabled(false);
    }
  }
}
